@@ -2,7 +2,7 @@
 ** ColorSpecialHandler.cpp                                              **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -31,9 +31,9 @@
 using namespace std;
 
 
-static float read_float (istream &is) {
+static double read_double (istream &is) {
 	is.clear();
-	float v;
+	double v;
 	is >> v;
 	if (is.fail())
 		throw SpecialException("number expected");
@@ -41,46 +41,46 @@ static float read_float (istream &is) {
 }
 
 
-/** Reads multiple float values from a given stream. The number of
+/** Reads multiple double values from a given stream. The number of
  *  values read is determined by the size of the result vector.
  *  @param[in]  is stream to be read from
- *  @param[out] v  the resulting floats */
-static void read_floats (istream &is, vector<float> &v) {
+ *  @param[out] v the resulting values */
+static void read_doubles (istream &is, vector<double> &v) {
 	for (size_t i=0; i < v.size(); i++)
-		v[i] = read_float(is);
+		v[i] = read_double(is);
 }
 
 
 /** Reads a color statement from an input stream and converts it to RGB.
  *  A color statement has the following syntax:
- *  <color model> <component values>
+ *  _color model_ _component values_
  *  Currently, the following color models are supported: rgb, cmyk, hsb and gray.
  *  Examples: rgb 1 0.5 0, gray 0.5
  *  @param[in]  model if model != "" this value specifies the model, otherwise it's read from the stream
  *  @param[in]  is stream to be read from
- *  @param[out] resulting RGB triple
+ *  @param[out] color italicresulting RGB triple
  *  @return true if statement has successfully been read */
 static void read_color (string model, istream &is, Color &color) {
 	if (model.empty())
 		is >> model;
 	if (model == "rgb") {
-		vector<float> rgb(3);
-		read_floats(is, rgb);
-		color.set(rgb[0], rgb[1], rgb[2]);
+		vector<double> rgb(3);
+		read_doubles(is, rgb);
+		color.setRGB(rgb[0], rgb[1], rgb[2]);
 	}
 	else if (model == "cmyk") {
-		vector<float> cmyk(4);
-		read_floats(is, cmyk);
+		vector<double> cmyk(4);
+		read_doubles(is, cmyk);
 		color.setCMYK(cmyk[0], cmyk[1], cmyk[2], cmyk[3]);
 	}
 	else if (model == "hsb") {
-		vector<float> hsb(3);
-		read_floats(is, hsb);
+		vector<double> hsb(3);
+		read_doubles(is, hsb);
 		color.setHSB(hsb[0], hsb[1], hsb[2]);
 	}
 	else if (model == "gray")
-		color.setGray(read_float(is));
-	else if (!color.set(model, true))
+		color.setGray(read_double(is));
+	else if (!color.setPSName(model, true))
 		throw SpecialException("unknown color statement");
 }
 
@@ -109,7 +109,7 @@ bool ColorSpecialHandler::process (const char *prefix, istream &is, SpecialActio
 			_colorStack.push(color);
 		}
 		if (actions) {
-		  	if (_colorStack.empty())
+			if (_colorStack.empty())
 				actions->setColor(Color::BLACK);
 			else
 				actions->setColor(_colorStack.top());
