@@ -2,7 +2,7 @@
 ** DVIReader.h                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -25,29 +25,17 @@
 #include <map>
 #include <stack>
 #include <string>
+#include "BasicDVIReader.h"
 #include "MessageException.h"
 #include "StreamReader.h"
 #include "VFActions.h"
 #include "types.h"
 
 
-struct DVIException : public MessageException
-{
-	DVIException (const std::string &msg) : MessageException(msg) {}
-};
-
-struct InvalidDVIFileException : public DVIException
-{
-	InvalidDVIFileException(const std::string &msg) : DVIException(msg) {}
-};
-
 struct DVIActions;
-struct FileFinder;
 
-class DVIReader : public StreamReader, protected VFActions
+class DVIReader : public BasicDVIReader, protected VFActions
 {
-	typedef void (DVIReader::*CommandHandler)(int);
-	enum DVIFormat {DVI_NONE=0, DVI_STANDARD=2, DVI_PTEX=3, DVI_XDV=5};
 	enum WritingMode {WMODE_LR=0, WMODE_TB=1, WMODE_BT=3};
 
 	struct DVIState
@@ -64,31 +52,30 @@ class DVIReader : public StreamReader, protected VFActions
 
 		bool executeDocument ();
 		void executeAll ();
-		bool executeAllPages ();
 		void executePreamble ();
 		void executePostamble ();
 		bool executePage (unsigned n);
-		bool executePages (unsigned first, unsigned last);
 		bool inPostamble () const              {return _inPostamble;}
 		double getXPos () const;
 		double getYPos () const;
 		void finishLine ()                     {_prevYPos = std::numeric_limits<double>::min();}
-		void translateToX (double x)           {_tx=x-_dviState.h-_tx;}
-		void translateToY (double y)           {_ty=y-_dviState.v-_ty;}
-		double getPageWidth () const;
-		double getPageHeight () const;
+		void translateToX (double x)           {_tx = x-_dviState.h-_tx;}
+		void translateToY (double y)           {_ty = y-_dviState.v-_ty;}
+		double getPageWidth () const           {return _pageWidth;}
+		double getPageHeight () const          {return _pageHeight;}
+		int getStackDepth () const             {return _stateStack.size();}
 		int getCurrentFontNumber () const      {return _currFontNum;}
 		unsigned getCurrentPageNumber () const {return _currPageNum;}
-		unsigned getTotalPages () const        {return _totalPages;}
+		unsigned numberOfPages () const        {return _bopOffsets.empty() ? 0 : _bopOffsets.size()-1;}
 		DVIActions* getActions () const        {return _actions;}
 		DVIActions* replaceActions (DVIActions *a);
 
 	protected:
-		void verifyDVIFormat (int id) const;
+		void collectBopOffsets ();
+		size_t numberOfPageBytes (int n) const {return _bopOffsets.size() > 1 ? _bopOffsets[n+1]-_bopOffsets[n] : 0;}
 		int executeCommand ();
-		int evalCommand (bool compute_size, CommandHandler &handler, int &length, int &param);
-		void moveRight (double x, bool callAction=true);
-		void moveDown (double y);
+		void moveRight (double dx);
+		void moveDown (double dy);
 		void putChar (UInt32 c, bool moveCursor);
 		void putGlyphArray (bool xonly);
 		void defineFont (UInt32 fontnum, const std::string &name, UInt32 cs, double ds, double ss);
@@ -131,30 +118,27 @@ class DVIReader : public StreamReader, protected VFActions
 		void cmdPostPost (int len);
 		void cmdXPic (int len);
 		void cmdXFontDef (int len);
-		void cmdXGlyphA (int len);
-		void cmdXGlyphS (int len);
+		void cmdXGlyphArray (int len);
+		void cmdXGlyphString (int len);
 
 	private:
-		DVIFormat _dviFormat;    ///< format of DVI file currently processed
 		DVIActions *_actions;    ///< actions to be performed on various DVI events
 		bool _inPage;            ///< true if between bop and eop
-		unsigned _totalPages;    ///< total number of pages in dvi file
-		unsigned _currPageNum;   ///< current page number
+		unsigned _currPageNum;   ///< current page number (1 is first page)
 		int _currFontNum;        ///< current font number
-		double _scaleFactor;     ///< 1 dvi unit = scaleFactor * TeX points
+		double _dvi2bp;          ///< factor to convert dvi units to PS points
 		UInt32 _mag;             ///< magnification factor * 1000
 		bool _inPostamble;       ///< true if stream pointer is inside the postamble
-		Int32 _prevBop;          ///< pointer to previous bop
-		double _pageHeight, _pageWidth;  ///< page height and width in TeX points
+		double _pageHeight, _pageWidth;  ///< page height and width in PS points
 		DVIState _dviState;      ///< current cursor position
 		std::stack<DVIState> _stateStack;
+		std::vector<UInt32> _bopOffsets;
 		double _prevYPos;        ///< previous vertical cursor position
 		double _tx, _ty;         ///< tranlation of cursor position
-		size_t _pageLength;      ///< number of bytes between current bop end eop
 		std::streampos _pagePos; ///< distance of current DVI command from bop (in bytes)
 
 	public:
-		static bool COMPUTE_PAGE_LENGTH;  ///< if true, the bop handler computes the number of bytes of the current page
+		static bool COMPUTE_PROGRESS;  ///< if true, an action to handle the progress ratio of a page is triggered
 };
 
 #endif
