@@ -2,7 +2,7 @@
 ** DVIToSVGActions.cpp                                                  **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -36,11 +36,8 @@
 using namespace std;
 
 
-double DVIToSVGActions::PROGRESSBAR_DELAY=1000;  // initial delay in seconds (values >= 1000 disable the progressbar)
-
-
 DVIToSVGActions::DVIToSVGActions (DVIToSVG &dvisvg, SVGTree &svg)
-	: _svg(svg), _dvisvg(dvisvg), _pageMatrix(0), _bgcolor(Color::TRANSPARENT), _boxes(0)
+	: _svg(svg), _dvireader(&dvisvg), _pageMatrix(0), _bgcolor(Color::TRANSPARENT), _boxes(0)
 {
 	_currentFontNum = -1;
 	_pageCount = 0;
@@ -69,22 +66,30 @@ void DVIToSVGActions::setPageMatrix (const Matrix &matrix) {
 
 
 void DVIToSVGActions::moveToX (double x) {
-	_dvisvg.specialManager().notifyPositionChange(getX(), getY());
-	_svg.setX(getX());        // get current horizontal DVI position
+	SpecialManager::instance().notifyPositionChange(getX(), getY());
+	_svg.setX(x);
 }
 
 
 void DVIToSVGActions::moveToY (double y) {
-	_dvisvg.specialManager().notifyPositionChange(getX(), getY());
+	SpecialManager::instance().notifyPositionChange(getX(), getY());
 	_svg.setY(y);
 }
 
 
+string DVIToSVGActions::getSVGFilename (unsigned pageno) const {
+	if (DVIToSVG *dvi2svg = dynamic_cast<DVIToSVG*>(_dvireader))
+		return dvi2svg->getSVGFilename(pageno);
+	return "";
+}
+
+
 /** This method is called when a "set char" command was found in the DVI file.
  *  It draws a character of the current font.
  *  @param[in] x horizontal position of left bounding box edge
  *  @param[in] y vertical position of the character's baseline
  *  @param[in] c character code relative to the current font
+ *  @param[in] vertical true if we're in vertical mode
  *  @param[in] font font to be used */
 void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, const Font *font) {
 	// If we use SVG fonts there is no need to record all font name/char/size combinations
@@ -162,10 +167,6 @@ void DVIToSVGActions::setChar (double x, double y, unsigned c, bool vertical, co
  *  @param[in] height length of the vertical edges
  *  @param[in] width length of the horizontal edges */
 void DVIToSVGActions::setRule (double x, double y, double height, double width) {
-/*	x *= BP;
-	y *= BP;
-	height *= BP;
-	width  *= BP; */
 	// (x,y) is the lower left corner of the rectangle
 	XMLElementNode *rect = new XMLElementNode("rect");
 	rect->addAttribute("x", x);
@@ -175,7 +176,7 @@ void DVIToSVGActions::setRule (double x, double y, double height, double width)
 	if (!getMatrix().isIdentity())
 		rect->addAttribute("transform", getMatrix().getSVG());
 	if (getColor() != Color::BLACK)
-		rect->addAttribute("fill", _svg.getColor().rgbString());
+		rect->addAttribute("fill", _svg.getColor().svgColorString());
 	_svg.appendToPage(rect);
 
 	// update bounding box
@@ -197,27 +198,23 @@ void DVIToSVGActions::setFont (int num, const Font *font) {
 
 
 /** This method is called when a "special" command was found in the DVI file.
- *  @param[in] s the special expression
- *  @param[in] dvi2pt factor to scale DVI units to TeX points */
-void DVIToSVGActions::special (const string &s, double dvi2pt) {
+ *  @param[in] spc the special expression
+ *  @param[in] dvi2bp factor to scale DVI units to PS points
+ *  @param[in] preprocessing if true, the DVI file is being pre-processed */
+void DVIToSVGActions::special (const string &spc, double dvi2bp, bool preprocessing) {
 	try {
-		_dvisvg.specialManager().process(s, dvi2pt, this, this);
+		if (preprocessing)
+			SpecialManager::instance().preprocess(spc, this);
+		else
+			SpecialManager::instance().process(spc, dvi2bp, this);
 		// @@ output message in case of unsupported specials?
 	}
 	catch (const SpecialException &e) {
-		Message::estream(true) << "error in special '" << s << "': " << e.what() << '\n';
+		Message::estream(true) << "error in special: " << e.what() << '\n';
 	}
 }
 
 
-void DVIToSVGActions::beginSpecial (const char *prefix) {
-}
-
-
-void DVIToSVGActions::endSpecial (const char *) {
-}
-
-
 /** This method is called when the DVI preamble was read
  *  @param[in] cmt preamble comment text. */
 void DVIToSVGActions::preamble (const string &cmt) {
@@ -229,7 +226,8 @@ void DVIToSVGActions::postamble () {
 
 
 /** This method is called when a "begin of page (bop)" command was found in the DVI file.
- *  @param[in] c array with 10 components representing \count0 ... \count9. c[0] contains the
+ *  @param[in] pageno physical page number
+ *  @param[in] c array with 10 components representing \\count0 ... \\count9. c[0] contains the
  *               current (printed) page number (may differ from page count) */
 void DVIToSVGActions::beginPage (unsigned pageno, Int32 *c) {
 	_svg.newPage(++_pageCount);
@@ -249,7 +247,7 @@ void DVIToSVGActions::endPage (unsigned pageno) {
 		r->addAttribute("y", _bbox.minY());
 		r->addAttribute("width", _bbox.width());
 		r->addAttribute("height", _bbox.height());
-		r->addAttribute("fill", _bgcolor.rgbString());
+		r->addAttribute("fill", _bgcolor.svgColorString());
 		_svg.prependToPage(r);
 	}
 }
