@@ -2,7 +2,7 @@
 ** DVIToSVGActions.h                                                    **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -37,24 +37,26 @@ struct FileFinder;
 struct Font;
 struct XMLNode;
 
-class DVIToSVGActions : public DVIActions, public SpecialActions, public SpecialManager::Listener
+class DVIToSVGActions : public DVIActions, public SpecialActions
 {
 	typedef std::map<const Font*, std::set<int> > CharMap;
 	typedef std::set<const Font*> FontSet;
-   typedef std::map<std::string,BoundingBox> BoxMap;
+	typedef std::map<std::string,BoundingBox> BoxMap;
 
 	public:
 		DVIToSVGActions (DVIToSVG &dvisvg, SVGTree &svg);
 		~DVIToSVGActions ();
-      void reset ();
+		void reset ();
 		void setChar (double x, double y, unsigned c, bool vertical, const Font *f);
 		void setRule (double x, double y, double height, double width);
 		void setBgColor (const Color &color);
 		void setColor (const Color &color)             {_svg.setColor(color);}
 		void setMatrix (const Matrix &m)               {_svg.setMatrix(m);}
 		const Matrix& getMatrix () const               {return _svg.getMatrix();}
-		void getPageTransform (Matrix &matrix) const   {_dvisvg.getPageTransformation(matrix);}
+		void getPageTransform (Matrix &matrix) const   {_dvireader->getPageTransformation(matrix);}
 		Color getColor () const                        {return _svg.getColor();}
+		int getDVIStackDepth() const                   {return _dvireader->getStackDepth();}
+		unsigned getCurrentPageNumber() const          {return _dvireader->getCurrentPageNumber();}
 		void appendToPage (XMLNode *node)              {_svg.appendToPage(node);}
 		void appendToDefs (XMLNode *node)              {_svg.appendToDefs(node);}
 		void prependToPage (XMLNode *node)             {_svg.prependToPage(node);}
@@ -64,34 +66,31 @@ class DVIToSVGActions : public DVIActions, public SpecialActions, public Special
 		void moveToX (double x);
 		void moveToY (double y);
 		void setFont (int num, const Font *font);
-		void special (const std::string &s, double dvi2pt);
+		void special (const std::string &spc, double dvi2bp, bool preprocessing=false);
 		void preamble (const std::string &cmt);
 		void postamble ();
 		void beginPage (unsigned pageno, Int32 *c);
 		void endPage (unsigned pageno);
-		void beginSpecial (const char *prefix);
-		void endSpecial (const char *prefix);
 		void progress (size_t current, size_t total, const char *id=0);
 		void progress (const char *id);
 		CharMap& getUsedChars () const        {return _usedChars;}
 		const FontSet& getUsedFonts () const  {return _usedFonts;}
+		void setDVIReader (BasicDVIReader &r) {_dvireader = &r;}
 		void setPageMatrix (const Matrix &tm);
-		double getX() const   {return _dvisvg.getXPos();}
-		double getY() const   {return _dvisvg.getYPos();}
-		void setX (double x)  {_dvisvg.translateToX(x); _svg.setX(x);}
-		void setY (double y)  {_dvisvg.translateToY(y); _svg.setY(y);}
-      void finishLine ()    {_dvisvg.finishLine();}
+		double getX() const   {return _dvireader->getXPos();}
+		double getY() const   {return _dvireader->getYPos();}
+		void setX (double x)  {_dvireader->translateToX(x); _svg.setX(x);}
+		void setY (double y)  {_dvireader->translateToY(y); _svg.setY(y);}
+		void finishLine ()    {_dvireader->finishLine();}
 		BoundingBox& bbox ()  {return _bbox;}
-      BoundingBox& bbox (const std::string &name, bool reset=false);
-      void embed (const BoundingBox &bbox);
-      void embed (const DPair &p, double r=0);
-
-	public:
-		static double PROGRESSBAR_DELAY; ///< progress bar doesn't appear before this time has elapsed (in sec)
+		BoundingBox& bbox (const std::string &name, bool reset=false);
+		void embed (const BoundingBox &bbox);
+		void embed (const DPair &p, double r=0);
+		std::string getSVGFilename (unsigned pageno) const;
 
 	private:
 		SVGTree &_svg;
-		DVIToSVG &_dvisvg;
+		BasicDVIReader *_dvireader;
 		BoundingBox _bbox;
 		int _pageCount;
 		int _currentFontNum;
@@ -99,7 +98,7 @@ class DVIToSVGActions : public DVIActions, public SpecialActions, public Special
 		FontSet _usedFonts;
 		Matrix *_pageMatrix;  // transformation of whole page
 		Color _bgcolor;
-      BoxMap *_boxes;
+		BoxMap *_boxes;
 };
 
 
