@@ -2,7 +2,7 @@
 ** EmSpecialHandler.cpp                                                 **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -30,7 +30,7 @@
 using namespace std;
 
 
-EmSpecialHandler::EmSpecialHandler () : _linewidth(0.4), _actions(0) {
+EmSpecialHandler::EmSpecialHandler () : _linewidth(0.4*72/72.27), _actions(0) {
 }
 
 
@@ -64,11 +64,12 @@ static DPair cut_vector (char c, const DPair &v, double lw) {
 
 
 /** Creates the SVG element that will a the line.
- * @param[in] p1 first endpoint in TeX point units
- * @param[in] p2 second endpoint in TeX point units
+ * @param[in] p1 first endpoint in PS point units
+ * @param[in] p2 second endpoint in PS point units
  * @param[in] c1 cut method of first endpoint ('h', 'v' or 'p')
  * @param[in] c2 cut method of second endpoint ('h', 'v' or 'p')
- * @param[in] lw line width in TeX point units */
+ * @param[in] lw line width in PS point units
+ * @param[in] actions object providing the actions that can be performed by the SpecialHandler */
 static void create_line (const DPair &p1, const DPair &p2, char c1, char c2, double lw, SpecialActions *actions) {
 	XMLElementNode *node=0;
 	DPair dir = p2-p1;
@@ -80,7 +81,7 @@ static void create_line (const DPair &p1, const DPair &p2, char c1, char c2, dou
 		node->addAttribute("x2", p2.x());
 		node->addAttribute("y2", p2.y());
 		node->addAttribute("stroke-width", lw);
-		node->addAttribute("stroke", actions->getColor().rgbString());
+		node->addAttribute("stroke", actions->getColor().svgColorString());
 		// update bounding box
 		actions->embed(p1);
 		actions->embed(p2);
@@ -94,12 +95,12 @@ static void create_line (const DPair &p1, const DPair &p2, char c1, char c2, dou
 		node = new XMLElementNode("polygon");
 		ostringstream oss;
 		oss << XMLString(q11.x()) << ',' << XMLString(q11.y()) << ' '
-		    << XMLString(q12.x()) << ',' << XMLString(q12.y()) << ' '
-		    << XMLString(q22.x()) << ',' << XMLString(q22.y()) << ' '
-		    << XMLString(q21.x()) << ',' << XMLString(q21.y());
+			 << XMLString(q12.x()) << ',' << XMLString(q12.y()) << ' '
+			 << XMLString(q22.x()) << ',' << XMLString(q22.y()) << ' '
+			 << XMLString(q21.x()) << ',' << XMLString(q21.y());
 		node->addAttribute("points", oss.str());
 		if (actions->getColor() != Color::BLACK)
-			node->addAttribute("fill", actions->getColor().rgbString());
+			node->addAttribute("fill", actions->getColor().svgColorString());
 		// update bounding box
 		actions->embed(q11);
 		actions->embed(q12);
@@ -134,7 +135,7 @@ static double read_length (InputReader &in) {
 			len *= p->factor;
 			break;
 		}
-	return len;
+	return len*72/72.27;
 }
 
 
