@@ -2,7 +2,7 @@
 ** Font.h                                                               **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -25,7 +25,6 @@
 #include <string>
 #include <vector>
 #include "Character.h"
-#include "CharMap.h"
 #include "CharMapID.h"
 #include "Color.h"
 #include "FontCache.h"
@@ -36,10 +35,11 @@
 #include "Glyph.h"
 #include "GraphicPath.h"
 #include "MessageException.h"
+#include "RangeMap.h"
+#include "ToUnicodeMap.h"
 #include "VFActions.h"
 #include "VFReader.h"
 #include "types.h"
-#include "CharMap.h"
 
 
 struct FontStyle;
@@ -68,11 +68,12 @@ struct Font {
 	virtual double italicCorr (int c) const =0;
 	virtual const FontMetrics* getMetrics () const =0;
 	virtual const char* path () const =0;
+	virtual const char* filename () const;
 	virtual const FontEncoding* encoding () const;
 	virtual bool getGlyph (int c, Glyph &glyph, GFGlyphTracer::Callback *cb=0) const =0;
 	virtual void getGlyphMetrics (int c, bool vertical, GlyphMetrics &metrics) const;
-   virtual UInt32 unicode (UInt32 c) const;
-   virtual void tidy () const {}
+	virtual UInt32 unicode (UInt32 c) const;
+	virtual void tidy () const {}
 	virtual bool findAndAssignBaseFontMap () {return true;}
 	virtual bool verticalLayout () const     {return getMetrics() ? getMetrics()->verticalLayout() : false;}
 	virtual bool verifyChecksums () const    {return true;}
@@ -111,8 +112,8 @@ struct EmptyFont : public Font
 /** Interface for all physical fonts. */
 class PhysicalFont : public virtual Font
 {
-   public:
-      enum Type {MF, OTF, PFB, TTC, TTF, UNKNOWN};
+	public:
+		enum Type {MF, OTF, PFB, TTC, TTF, UNKNOWN};
 
 		static Font* create (std::string name, UInt32 checksum, double dsize, double ssize, PhysicalFont::Type type);
 		static Font* create (std::string name, int fontindex, UInt32 checksum, double dsize, double ssize);
@@ -136,16 +137,16 @@ class PhysicalFont : public virtual Font
 		virtual Character decodeChar (UInt32 c) const;
 		const char* path () const;
 
-   protected:
+	protected:
 		bool createGF (std::string &gfname) const;
 
-   public:
+	public:
 		static bool EXACT_BBOX;
 		static bool KEEP_TEMP_FILES;
 		static const char *CACHE_PATH; ///< path to cache directory (0 if caching is disabled)
 		static double METAFONT_MAG;    ///< magnification factor for Metafont calls
 
-   protected:
+	protected:
 		static FontCache _cache;
 };
 
@@ -186,8 +187,8 @@ class TFMFont : public virtual Font
 		mutable FontMetrics *_metrics;
 		std::string _fontname;
 		UInt32 _checksum; ///< cheksum to be compared with TFM checksum
-		double _dsize;    ///< design size in TeX point units
-		double _ssize;    ///< scaled size
+		double _dsize;    ///< design size in PS point units
+		double _ssize;    ///< scaled size in PS point units
 };
 
 
@@ -220,8 +221,8 @@ class PhysicalFontProxy : public PhysicalFont
 
 	private:
 		const PhysicalFont *_pf;
-		double _dsize;  ///< design size in TeX point units
-		double _ssize;  ///< scaled size
+		double _dsize;  ///< design size in PS point units
+		double _ssize;  ///< scaled size in PS point units
 };
 
 
@@ -229,17 +230,17 @@ class PhysicalFontImpl : public PhysicalFont, public TFMFont
 {
 	friend class PhysicalFont;
 	public:
-      ~PhysicalFontImpl();
+		~PhysicalFontImpl();
 		Font* clone (double ds, double ss) const    {return new PhysicalFontProxy(this, ds, ss);}
 		const Font* uniqueFont () const             {return this;}
-      Type type () const                          {return _filetype;}
+		Type type () const                          {return _filetype;}
 		int fontIndex() const                       {return _fontIndex;}
 		const FontStyle* style () const             {return _fontMapEntry ? &_fontMapEntry->style : 0;}
 		const FontMap::Entry* fontMapEntry () const {return _fontMapEntry;}
 		const FontEncoding* encoding () const;
-      UInt32 unicode (UInt32 c) const;
+		UInt32 unicode (UInt32 c) const;
 		bool findAndAssignBaseFontMap ();
-      void tidy () const;
+		void tidy () const;
 		CharMapID getCharMapID () const             {return _charmapID;}
 
 	protected:
@@ -251,7 +252,7 @@ class PhysicalFontImpl : public PhysicalFont, public TFMFont
 		const FontMap::Entry *_fontMapEntry;
 		FontEncodingPair _encodingPair;
 		CharMapID _charmapID;  ///< ID of the font's charmap to use
-		const CharMap *_localCharMap;
+		const RangeMap *_localCharMap;
 };
 
 
@@ -262,7 +263,6 @@ class NativeFont : public PhysicalFont
 		virtual Font* clone (double ds, double sc) const =0;
 		std::string name () const;
 		Type type () const;
-		UInt32 unicode (UInt32 c) const;
 		double designSize () const               {return _ptsize;}
 		double scaledSize () const               {return _ptsize;}
 		double charWidth (int c) const;
@@ -279,7 +279,7 @@ class NativeFont : public PhysicalFont
 		NativeFont (double ptsize, const FontStyle &style, Color color) : _ptsize(ptsize), _style(style), _color(color) {}
 
 	private:
-		double _ptsize;
+		double _ptsize;    ///< font size in PS point units
 		FontStyle _style;
 		Color _color;
 };
@@ -296,7 +296,9 @@ class NativeFontProxy : public NativeFont
 		Font* clone (double ds, double sc) const {return new NativeFontProxy(this , sc, *style(), color());}
 		const Font* uniqueFont () const          {return _nfont;}
 		const char* path () const                {return _nfont->path();}
+		int fontIndex () const                   {return _nfont->fontIndex();}
 		Character decodeChar (UInt32 c) const    {return _nfont->decodeChar(c);}
+		UInt32 unicode (UInt32 c) const          {return _nfont->unicode(c);}
 		CharMapID getCharMapID () const          {return _nfont->getCharMapID();}
 
 	protected:
@@ -311,8 +313,8 @@ class NativeFontProxy : public NativeFont
 class NativeFontImpl : public NativeFont
 {
 	public:
-		NativeFontImpl (const std::string &fname, double ptsize, const FontStyle &style, Color color)
-			: NativeFont(ptsize, style, color), _path(fname) {}
+		NativeFontImpl (const std::string &fname, int fontIndex, double ptsize, const FontStyle &style, Color color)
+			: NativeFont(ptsize, style, color), _path(fname), _fontIndex(fontIndex) {}
 
 		NativeFont* clone (double ptsize, const FontStyle &style, Color color) const {
 			return new NativeFontProxy(this, ptsize, style, color);
@@ -321,14 +323,16 @@ class NativeFontImpl : public NativeFont
 		Font* clone (double ds, double sc) const {return new NativeFontProxy(this , sc, *style(), color());}
 		const Font* uniqueFont () const          {return this;}
 		const char* path () const                {return _path.c_str();}
+		int fontIndex() const                    {return _fontIndex;}
 		bool findAndAssignBaseFontMap ();
-		CharMapID getCharMapID () const          {return _charmapID;}
+		CharMapID getCharMapID () const          {return CharMapID::NONE;}
 		Character decodeChar (UInt32 c) const;
+		UInt32 unicode (UInt32 c) const;
 
 	private:
 		std::string _path;
-		CharMap _toUnicodeMap;  ///< maps from char indexes to unicode point
-		CharMapID _charmapID;   ///< unicode charmap ID
+		int _fontIndex;
+		ToUnicodeMap _toUnicodeMap; ///< maps from char indexes to unicode points
 };
 
 
@@ -356,8 +360,8 @@ class VirtualFontProxy : public VirtualFont
 
 	private:
 		const VirtualFont *_vf;
-		double _dsize;  ///< design size in TeX point units
-		double _ssize;  ///< scaled size in TeX point units
+		double _dsize;  ///< design size in PS point units
+		double _ssize;  ///< scaled size in PS point units
 };
 
 
