@@ -2,7 +2,7 @@
 ** FontEncoding.h                                                       **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -35,7 +35,7 @@ struct FontEncoding
 	virtual Character decode (UInt32 c) const =0;
 	virtual bool mapsToCharIndex () const =0;
 	virtual const FontEncoding* findCompatibleBaseFontMap (const PhysicalFont *font, CharMapID &charmapID) const {return 0;}
-	static FontEncoding* encoding (const std::string &fontname);
+	static FontEncoding* encoding (const std::string &encname);
 };
 
 
