@@ -2,7 +2,7 @@
 ** FontManager.cpp                                                      **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -31,7 +31,6 @@
 #include "FileSystem.h"
 #include "Message.h"
 #include "macros.h"
-#include "CMapManager.h"
 
 using namespace std;
 
@@ -44,8 +43,8 @@ FontManager::~FontManager () {
 
 /** Returns the singleton instance */
 FontManager& FontManager::instance () {
-   static FontManager fm;
-   return fm;
+	static FontManager fm;
+	return fm;
 }
 
 
@@ -174,8 +173,8 @@ static Font* create_font (const string &filename, const string &fontname, int fo
  *  @param[in] fontnum local font number, as used in DVI and VF files
  *  @param[in] name TFM fontname given in DVI file, e.g. cmr10
  *  @param[in] checksum checksum to be compared with TFM checksum
- *  @param[in] dsize design size in TeX point units
- *  @param[in] ssize scaled size in TeX point units
+ *  @param[in] dsize design size in PS point units
+ *  @param[in] ssize scaled size in PS point units
  *  @return id of registered font */
 int FontManager::registerFont (UInt32 fontnum, string name, UInt32 checksum, double dsize, double ssize) {
 	int id = fontID(fontnum);
@@ -239,8 +238,12 @@ int FontManager::registerFont (UInt32 fontnum, string name, UInt32 checksum, dou
 }
 
 
-
 int FontManager::registerFont (UInt32 fontnum, string filename, double ptsize, const FontStyle &style, Color color) {
+	return registerFont(fontnum, filename, 0, ptsize, style, color);
+}
+
+
+int FontManager::registerFont (UInt32 fontnum, string filename, int fontIndex, double ptsize, const FontStyle &style, Color color) {
 	int id = fontID(fontnum);
 	if (id >= 0)
 		return id;
@@ -260,7 +263,7 @@ int FontManager::registerFont (UInt32 fontnum, string filename, double ptsize, c
 		if (!FileSystem::exists(path))
 			path = FileFinder::lookup(filename, false);
 		if (path) {
-			newfont = new NativeFontImpl(path, ptsize, style, color);
+			newfont = new NativeFontImpl(path, fontIndex, ptsize, style, color);
 			newfont->findAndAssignBaseFontMap();
 		}
 		if (!newfont) {
