@@ -2,7 +2,7 @@
 ** HtmlSpecialHandler.cpp                                               **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -19,6 +19,7 @@
 *************************************************************************/
 
 #include <config.h>
+#include <cassert>
 #include <sstream>
 #include "HtmlSpecialHandler.h"
 #include "InputReader.h"
@@ -29,8 +30,57 @@
 using namespace std;
 
 // variable to select the link marker variant (none, underlined, boxed, or colored background)
-HtmlSpecialHandler::LinkMarker HtmlSpecialHandler::LINK_MARKER = HtmlSpecialHandler::LM_LINE;
+HtmlSpecialHandler::MarkerType HtmlSpecialHandler::MARKER_TYPE = HtmlSpecialHandler::MT_LINE;
 Color HtmlSpecialHandler::LINK_BGCOLOR;
+Color HtmlSpecialHandler::LINK_LINECOLOR;
+bool HtmlSpecialHandler::USE_LINECOLOR = false;
+
+
+void HtmlSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActions *actions) {
+	if (!actions)
+		return;
+	_actions = actions;
+	StreamInputReader ir(is);
+	ir.skipSpace();
+	// collect page number and ID of named anchors
+	map<string,string> attribs;
+	if (ir.check("<a ") && ir.parseAttributes(attribs, '"') > 0) {
+		map<string,string>::iterator it;
+		if ((it = attribs.find("name")) != attribs.end())
+			preprocessNameAnchor(it->second);
+		else if ((it = attribs.find("href")) != attribs.end())
+			preprocessHrefAnchor(it->second);
+	}
+}
+
+
+void HtmlSpecialHandler::preprocessNameAnchor (const string &name) {
+	NamedAnchors::iterator it = _namedAnchors.find(name);
+	if (it == _namedAnchors.end()) {  // anchor completely undefined?
+		int id = _namedAnchors.size()+1;
+		_namedAnchors[name] = NamedAnchor(_actions->getCurrentPageNumber(), id, 0);
+	}
+	else if (it->second.id < 0) {  // anchor referenced but not defined yet?
+		it->second.id *= -1;
+		it->second.pageno = _actions->getCurrentPageNumber();
+	}
+	else
+		Message::wstream(true) << "named hyperref anchor '" << name << "' redefined\n";
+}
+
+
+void HtmlSpecialHandler::preprocessHrefAnchor (const string &uri) {
+	if (uri[0] != '#')
+		return;
+	string name = uri.substr(1);
+	NamedAnchors::iterator it = _namedAnchors.find(name);
+	if (it != _namedAnchors.end())  // anchor already defined?
+		it->second.referenced = true;
+	else {
+		int id = _namedAnchors.size()+1;
+		_namedAnchors[name] = NamedAnchor(0, -id, 0, true);
+	}
+}
 
 
 bool HtmlSpecialHandler::process (const char *prefix, istream &is, SpecialActions *actions) {
@@ -43,9 +93,9 @@ bool HtmlSpecialHandler::process (const char *prefix, istream &is, SpecialAction
 	map<string,string>::iterator it;
 	if (ir.check("<a ") && ir.parseAttributes(attribs, '"') > 0) {
 		if ((it = attribs.find("href")) != attribs.end())   // <a href="URI">
-			hrefAnchor(it->second);
+			processHrefAnchor(it->second);
 		else if ((it = attribs.find("name")) != attribs.end())  // <a name="ID">
-			nameAnchor(it->second);
+			processNameAnchor(it->second);
 		else
 			return false;  // none or only invalid attributes
 	}
@@ -61,15 +111,22 @@ bool HtmlSpecialHandler::process (const char *prefix, istream &is, SpecialAction
 
 /** Handles anchors with href attribute: <a href="URI">...</a>
  *  @param uri value of href attribute */
-void HtmlSpecialHandler::hrefAnchor (string uri) {
+void HtmlSpecialHandler::processHrefAnchor (string uri) {
 	closeAnchor();
-	int id=0;
+	string name;
 	if (uri[0] == '#') {  // reference to named anchor?
-		string name = uri.substr(1);
+		name = uri.substr(1);
 		NamedAnchors::iterator it = _namedAnchors.find(name);
-		if (it == _namedAnchors.end()) {
-			id = _namedAnchors.size()+1;
-			_namedAnchors[name] = NamedAnchor(-id, 0);
+		if (it == _namedAnchors.end() || it->second.id < 0)
+			Message::wstream(true) << "reference to undefined anchor '" << name << "'\n";
+		else {
+			int id = it->second.id;
+			uri = "#loc"+XMLString(id);
+			if (_actions->getCurrentPageNumber() != it->second.pageno) {
+				ostringstream oss;
+				oss << _actions->getSVGFilename(it->second.pageno) << uri;
+				uri = oss.str();
+			}
 		}
 	}
 	if (!_base.empty() && uri.find("://") != string::npos) {
@@ -78,31 +135,22 @@ void HtmlSpecialHandler::hrefAnchor (string uri) {
 		uri = _base + uri;
 	}
 	XMLElementNode *anchor = new XMLElementNode("a");
-	anchor->addAttribute("xlink:href", id ? "#loc"+XMLString(id) : uri);
-	anchor->addAttribute("xlink:title", uri);
+	anchor->addAttribute("xlink:href", uri);
+	anchor->addAttribute("xlink:title", XMLString(name.empty() ? uri : name, false));
 	_actions->pushContextElement(anchor);
 	_actions->bbox("{anchor}", true);  // start computing the bounding box of the linked area
-	_anchorYPos = _actions->getY();
+	_depthThreshold = _actions->getDVIStackDepth();
 	_anchorType = AT_HREF;
 }
 
 
 /** Handles anchors with name attribute: <a name="NAME">...</a>
  *  @param name value of name attribute */
-void HtmlSpecialHandler::nameAnchor (const string &name) {
+void HtmlSpecialHandler::processNameAnchor (const string &name) {
 	closeAnchor();
-	_anchorName = name;
 	NamedAnchors::iterator it = _namedAnchors.find(name);
-	if (it == _namedAnchors.end()) { // first occurrence of this anchor
-		int id = _namedAnchors.size()+1;
-		_namedAnchors[name] = NamedAnchor(id, _actions->getY());
-	}
-	else if (it->second.id < 0) {   // anchor referenced but not defined yet
-		it->second.id *= -1;
-		it->second.pos = _actions->getY();
-	}
-	else
-		Message::wstream(true) << "named hyperref anchor '" << name << "' redefined\n";
+	assert(it != _namedAnchors.end());
+	it->second.pos = _actions->getY();
 	_anchorType = AT_NAME;
 }
 
@@ -112,9 +160,8 @@ void HtmlSpecialHandler::closeAnchor () {
 	if (_anchorType == AT_HREF) {
 		markLinkedBox();
 		_actions->popContextElement();
+		_depthThreshold = 0;
 	}
-	else if (_anchorType == AT_NAME)
-		_anchorName.clear();
 	_anchorType = AT_NONE;
 }
 
@@ -126,25 +173,31 @@ void HtmlSpecialHandler::closeAnchor () {
 void HtmlSpecialHandler::markLinkedBox () {
 	const BoundingBox &bbox = _actions->bbox("{anchor}");
 	if (bbox.width() > 0 && bbox.height() > 0) {  // does the bounding box extend in both dimensions?
-		if (LINK_MARKER != LM_NONE) {
+		if (MARKER_TYPE != MT_NONE) {
 			const double linewidth = min(0.5, bbox.height()/15);
 			XMLElementNode *rect = new XMLElementNode("rect");
 			double x = bbox.minX();
 			double y = bbox.maxY()+linewidth;
 			double w = bbox.width();
 			double h = linewidth;
-			if (LINK_MARKER == LM_LINE)
-				rect->addAttribute("fill", _actions->getColor().rgbString());
+			const Color &linecolor = USE_LINECOLOR ? LINK_LINECOLOR : _actions->getColor();
+			if (MARKER_TYPE == MT_LINE)
+				rect->addAttribute("fill", linecolor.svgColorString());
 			else {
 				x -= linewidth;
 				y = bbox.minY()-linewidth;
 				w += 2*linewidth;
 				h += bbox.height()+linewidth;
-				if (LINK_MARKER == LM_BGCOLOR)
-					rect->addAttribute("fill", LINK_BGCOLOR.rgbString());
+				if (MARKER_TYPE == MT_BGCOLOR) {
+					rect->addAttribute("fill", LINK_BGCOLOR.svgColorString());
+					if (USE_LINECOLOR) {
+						rect->addAttribute("stroke", linecolor.svgColorString());
+						rect->addAttribute("stroke-width", linewidth);
+					}
+				}
 				else {  // LM_BOX
 					rect->addAttribute("fill", "none");
-					rect->addAttribute("stroke", _actions->getColor().rgbString());
+					rect->addAttribute("stroke", linecolor.svgColorString());
 					rect->addAttribute("stroke-width", linewidth);
 				}
 			}
@@ -153,7 +206,8 @@ void HtmlSpecialHandler::markLinkedBox () {
 			rect->addAttribute("width", w);
 			rect->addAttribute("height", h);
 			_actions->prependToPage(rect);
-			if (LINK_MARKER == LM_BOX) {
+			if (MARKER_TYPE == MT_BOX || MARKER_TYPE == MT_BGCOLOR) {
+				// slightly enlarge the boxed area
 				x -= linewidth;
 				y -= linewidth;
 				w += 2*linewidth;
@@ -180,9 +234,11 @@ void HtmlSpecialHandler::markLinkedBox () {
 /** This method is called every time the DVI position changes. */
 void HtmlSpecialHandler::dviMovedTo (double x, double y) {
 	if (_actions && _anchorType != AT_NONE) {
-		if (y != _anchorYPos) {  // does vertical position changed inside a linked area?
+		// Start a new box if the current depth of the DVI stack underruns
+		// the initial threshold which indicates a line break.
+		if (_actions->getDVIStackDepth() < _depthThreshold) {
 			markLinkedBox();
-			_anchorYPos = y;
+			_depthThreshold = _actions->getDVIStackDepth();
 			_actions->bbox("{anchor}", true);  // start a new box on the new line
 		}
 	}
@@ -194,7 +250,7 @@ void HtmlSpecialHandler::dviEndPage (unsigned pageno) {
 		// create views for all collected named anchors defined on the recent page
 		const BoundingBox &pagebox = _actions->bbox();
 		for (NamedAnchors::iterator it=_namedAnchors.begin(); it != _namedAnchors.end(); ++it) {
-			if (it->second.id > 0) {  // current anchor referenced?
+			if (it->second.pageno == pageno && it->second.referenced) {  // current anchor referenced?
 				ostringstream oss;
 				oss << pagebox.minX() << ' ' << it->second.pos << ' '
 					 << pagebox.width() << ' ' << pagebox.height();
@@ -205,23 +261,40 @@ void HtmlSpecialHandler::dviEndPage (unsigned pageno) {
 			}
 		}
 		closeAnchor();
-		_namedAnchors.clear();
 		_actions = 0;
 	}
 }
 
 
-bool HtmlSpecialHandler::setLinkMarker (const string &type) {
-	if (type.empty() || type == "none")
-		LINK_MARKER = LM_NONE;
-	else if (type == "line")
-		LINK_MARKER = LM_LINE;
-	else if (type == "box")
-		LINK_MARKER = LM_BOX;
+/** Sets the appearance of the link markers.
+ *  @param[in] marker string specifying the marker (format: type[:linecolor])
+ *  @return true on success */
+bool HtmlSpecialHandler::setLinkMarker (const string &marker) {
+	string type;  // "none", "box", "line", or a background color specifier
+	string color; // optional line color specifier
+	size_t seppos = marker.find(":");
+	if (seppos == string::npos)
+		type = marker;
 	else {
-		if (!LINK_BGCOLOR.set(type, false))
+		type = marker.substr(0, seppos);
+		color = marker.substr(seppos+1);
+	}
+	if (type.empty() || type == "none")
+		MARKER_TYPE = MT_NONE;
+	else if (type == "line")
+		MARKER_TYPE = MT_LINE;
+	else if (type == "box")
+		MARKER_TYPE = MT_BOX;
+	else {
+		if (!LINK_BGCOLOR.setPSName(type, false))
 			return false;
-		LINK_MARKER = LM_BGCOLOR;
+		MARKER_TYPE = MT_BGCOLOR;
+	}
+	USE_LINECOLOR = false;
+	if (MARKER_TYPE != MT_NONE && !color.empty()) {
+		if (!LINK_LINECOLOR.setPSName(color, false))
+			return false;
+		USE_LINECOLOR = true;
 	}
 	return true;
 }
