@@ -2,7 +2,7 @@
 ** PSPattern.cpp                                                        **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -47,10 +47,6 @@ void PSPattern::apply (SpecialActions *actions) {
 PSTilingPattern::PSTilingPattern (int id, BoundingBox &bbox, Matrix &matrix, double xstep, double ystep)
 	: PSPattern(id), _bbox(bbox), _matrix(matrix), _xstep(xstep), _ystep(ystep), _groupNode(0)
 {
-	const double bp2pt = 72.27/72.0;  // factor to convert bp to pt
-	_bbox.scale(bp2pt, bp2pt);
-	_xstep *= bp2pt;
-	_ystep *= bp2pt;
 	_groupNode = PSTilingPattern::createGroupNode();
 }
 
@@ -158,10 +154,10 @@ void PSUncoloredTilingPattern::apply (SpecialActions* actions) {
 		vector<XMLElementNode*> colored_elems;
 		const char *attribs[] = {"fill", "stroke"};
 		for (int i=0; i < 2; i++) {
-			getGroupNode()->findDescendants(0, attribs[i], colored_elems);
+			getGroupNode()->getDescendants(0, attribs[i], colored_elems);
 			for (vector<XMLElementNode*>::iterator it=colored_elems.begin(); it != colored_elems.end(); ++it)
 				if (string((*it)->getAttributeValue(attribs[i])) != "none")
-					(*it)->addAttribute(attribs[i], _currentColor.rgbString());
+					(*it)->addAttribute(attribs[i], _currentColor.svgColorString());
 			colored_elems.clear();
 		}
 		PSPattern::apply(actions);
