@@ -2,7 +2,7 @@
 ** PsSpecialHandler.cpp                                                 **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -22,20 +22,28 @@
 #include <cmath>
 #include <fstream>
 #include <iostream>
+#include <memory>
 #include <sstream>
 #include "EPSFile.h"
 #include "FileFinder.h"
 #include "Ghostscript.h"
 #include "Message.h"
+#include "PathClipper.h"
 #include "PSPattern.h"
 #include "PSPreviewFilter.h"
 #include "PsSpecialHandler.h"
 #include "SpecialActions.h"
+#include "SVGTree.h"
+#include "TensorProductPatch.h"
+#include "VectorIterator.h"
 #include "XMLNode.h"
 #include "XMLString.h"
+#include "TriangularPatch.h"
+
 
 using namespace std;
 
+
 static inline double str2double (const string &str) {
 	double ret;
 	istringstream iss(str);
@@ -44,6 +52,12 @@ static inline double str2double (const string &str) {
 }
 
 
+bool PsSpecialHandler::COMPUTE_CLIPPATHS_INTERSECTIONS = false;
+bool PsSpecialHandler::SHADING_SEGMENT_OVERLAP = false;
+int PsSpecialHandler::SHADING_SEGMENT_SIZE = 20;
+double PsSpecialHandler::SHADING_SIMPLIFY_DELTA = 0.01;
+
+
 PsSpecialHandler::PsSpecialHandler () : _psi(this), _actions(0), _previewFilter(_psi), _psSection(PS_NONE), _xmlnode(0)
 {
 }
@@ -74,6 +88,9 @@ void PsSpecialHandler::initialize () {
 		const char *headers[] = {"tex.pro", "texps.pro", "special.pro", /*"color.pro",*/ 0};
 		for (const char **p=headers; *p; ++p)
 			processHeaderFile(*p);
+		// disable bop/eop operators to prevent side-effects by
+		// unexpected bobs/eops present in PS specials
+		_psi.execute("\nTeXDict begin /bop{pop pop}def /eop{}def end ");
 		_psSection = PS_HEADERS;  // allow to process header specials now
 	}
 }
@@ -103,11 +120,6 @@ void PsSpecialHandler::enterBodySection () {
 		// push dictionary "TeXDict" with dvips definitions on dictionary stack
 		// and initialize basic dvips PostScript variables
 		oss << "\nTeXDict begin 0 0 1000 72 72 () @start 0 0 moveto ";
-  		if (_actions) {
-			float r, g, b;
-			_actions->getColor().getRGB(r, g, b);
-			oss << r << ' ' << g << ' ' << b << " setrgbcolor ";
-		}
 		_psi.execute(oss.str(), false);
 		// Check for information generated by preview.sty. If the tightpage options
 		// was set, don't execute the bop-hook but allow the PS filter to read
@@ -123,62 +135,77 @@ void PsSpecialHandler::enterBodySection () {
 /** Move PS graphic position to current DVI location. */
 void PsSpecialHandler::moveToDVIPos () {
 	if (_actions) {
-		const double bp=72.0/72.27; // pt -> bp
-		const double x = _actions->getX()*bp;
-		const double y = _actions->getY()*bp;
+		const double x = _actions->getX();
+		const double y = _actions->getY();
 		ostringstream oss;
-      oss << '\n' << x << ' ' << y << " moveto ";
-      _psi.execute(oss.str());
-      _currentpoint = DPair(x, y);
-   }
-}
-
-
-/** Executes a PS snippet and moves the DVI cursor to the current DVI position afterwards.
- *  It's just a shorthand function as this action sequence is required several times.
- *  @param[in] psi PS interpreter instance
- *  @param[in] is  stream to read the PS code from
- *  @param[in] pos current PS graphic position
- *  @param[in] actions special actions */
-static void exec_and_syncpos (PSInterpreter &psi, istream &is, const DPair &pos, SpecialActions *actions) {
-	psi.execute(is);
-	psi.execute("\nquerypos ");   // retrieve current PS position (stored in 'pos')
-	if (actions) {
-		const double pt = 72.27/72.0; // bp -> pt
-		actions->setX(pos.x()*pt);
-		actions->setY(pos.y()*pt);
+		oss << '\n' << x << ' ' << y << " moveto ";
+		_psi.execute(oss.str());
+		_currentpoint = DPair(x, y);
 	}
 }
 
 
+/** Executes a PS snippet and optionally synchronizes the DVI cursor position
+ *  with the current PS point.
+ *  @param[in] is  stream to read the PS code from
+ *  @param[in] updatePos if true, move the DVI drawing position to the current PS point */
+void PsSpecialHandler::executeAndSync (istream &is, bool updatePos) {
+	if (_actions && _actions->getColor() != _currentcolor) {
+		// update the PS graphics state if the color has been changed by a color special
+		double r, g, b;
+		_actions->getColor().getRGB(r, g, b);
+		ostringstream oss;
+		oss << '\n' << r << ' ' << g << ' ' << b << " setrgbcolor ";
+		_psi.execute(oss.str(), false);
+	}
+	_psi.execute(is);
+	if (updatePos) {
+		// retrieve current PS position (stored in _currentpoint)
+		_psi.execute("\nquerypos ");
+		if (_actions) {
+			_actions->setX(_currentpoint.x());
+			_actions->setY(_currentpoint.y());
+		}
+	}
+}
+
+
+void PsSpecialHandler::preprocess (const char *prefix, istream &is, SpecialActions *actions) {
+	initialize();
+	if (_psSection != PS_HEADERS)
+		return;
+
+	_actions = actions;
+	if (*prefix == '!') {
+		_headerCode += "\n";
+		_headerCode += string(istreambuf_iterator<char>(is), istreambuf_iterator<char>());
+	}
+	else if (strcmp(prefix, "header=") == 0) {
+		// read and execute PS header file
+		string fname;
+		is >> fname;
+		processHeaderFile(fname.c_str());
+	}
+}
+
 
 bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions *actions) {
+	// process PS headers only once (in prescan)
+	if (*prefix == '!' || strcmp(prefix, "header=") == 0)
+		return true;
+
 	_actions = actions;
 	initialize();
-	if (_psSection != PS_BODY && *prefix != '!' && strcmp(prefix, "header=") != 0)
+	if (_psSection != PS_BODY)
 		enterBodySection();
 
 	if (*prefix == '"') {
 		// read and execute literal PostScript code (isolated by a wrapping save/restore pair)
 		moveToDVIPos();
 		_psi.execute("\n@beginspecial @setspecial ");
-		_psi.execute(is);
+		executeAndSync(is, false);
 		_psi.execute("\n@endspecial ");
 	}
-	else if (*prefix == '!') {
-		if (_psSection == PS_HEADERS) {
-			_headerCode += "\n";
-			_headerCode += string(istreambuf_iterator<char>(is), istreambuf_iterator<char>());
-		}
-	}
-	else if (strcmp(prefix, "header=") == 0) {
-		if (_psSection == PS_HEADERS) {
-			// read and execute PS header file
-			string fname;
-			is >> fname;
-			processHeaderFile(fname.c_str());
-		}
-	}
 	else if (strcmp(prefix, "psfile=") == 0 || strcmp(prefix, "PSfile=") == 0) {
 		if (_actions) {
 			StreamInputReader in(is);
@@ -201,18 +228,18 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 
 			if (code == "[begin]" || code == "[nobreak]") {
 				moveToDVIPos();
-				exec_and_syncpos(_psi, is, _currentpoint, _actions);
+				executeAndSync(is, true);
 			}
 			else {
 				// no move to DVI position here
 				if (code != "[end]") // PS array?
 					_psi.execute(code);
-				exec_and_syncpos(_psi, is, _currentpoint, _actions);
+				executeAndSync(is, true);
 			}
 		}
 		else { // ps::<code> behaves like ps::[end]<code>
 			// no move to DVI position here
-			exec_and_syncpos(_psi, is, _currentpoint, _actions);
+			executeAndSync(is, true);
 		}
 	}
 	else { // ps: ...
@@ -231,7 +258,7 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 		else {
 			// ps:<code> is almost identical to ps::[begin]<code> but does
 			// a final repositioning to the current DVI location
-			exec_and_syncpos(_psi, is, _currentpoint, _actions);
+			executeAndSync(is, true);
 			moveToDVIPos();
 		}
 	}
@@ -241,7 +268,7 @@ bool PsSpecialHandler::process (const char *prefix, istream &is, SpecialActions
 
 /** Handles psfile special.
  *  @param[in] fname EPS file to be included
- *  @param[in] attr attributes given with \special psfile */
+ *  @param[in] attr attributes given with \\special psfile */
 void PsSpecialHandler::psfile (const string &fname, const map<string,string> &attr) {
 	EPSFile epsfile(fname);
 	istream &is = epsfile.istream();
@@ -249,23 +276,22 @@ void PsSpecialHandler::psfile (const string &fname, const map<string,string> &at
 		Message::wstream(true) << "file '" << fname << "' not found in special 'psfile'\n";
 	else {
 		map<string,string>::const_iterator it;
-		const double pt = 72.27/72.0;  // bp -> pt
 
 		// bounding box of EPS figure
-		double llx = (it = attr.find("llx")) != attr.end() ? str2double(it->second)*pt : 0;
-		double lly = (it = attr.find("lly")) != attr.end() ? str2double(it->second)*pt : 0;
-		double urx = (it = attr.find("urx")) != attr.end() ? str2double(it->second)*pt : 0;
-		double ury = (it = attr.find("ury")) != attr.end() ? str2double(it->second)*pt : 0;
+		double llx = (it = attr.find("llx")) != attr.end() ? str2double(it->second) : 0;
+		double lly = (it = attr.find("lly")) != attr.end() ? str2double(it->second) : 0;
+		double urx = (it = attr.find("urx")) != attr.end() ? str2double(it->second) : 0;
+		double ury = (it = attr.find("ury")) != attr.end() ? str2double(it->second) : 0;
 
 		// desired width/height of resulting figure
-		double rwi = (it = attr.find("rwi")) != attr.end() ? str2double(it->second)/10.0*pt : -1;
-		double rhi = (it = attr.find("rhi")) != attr.end() ? str2double(it->second)/10.0*pt : -1;
+		double rwi = (it = attr.find("rwi")) != attr.end() ? str2double(it->second)/10.0 : -1;
+		double rhi = (it = attr.find("rhi")) != attr.end() ? str2double(it->second)/10.0 : -1;
 		if (rwi == 0 || rhi == 0 || urx-llx == 0 || ury-lly == 0)
 			return;
 
 		// user transformations (default values chosen according to dvips manual)
-		double hoffset = (it = attr.find("hoffset")) != attr.end() ? str2double(it->second)*pt : 0;
-		double voffset = (it = attr.find("voffset")) != attr.end() ? str2double(it->second)*pt : 0;
+		double hoffset = (it = attr.find("hoffset")) != attr.end() ? str2double(it->second) : 0;
+		double voffset = (it = attr.find("voffset")) != attr.end() ? str2double(it->second) : 0;
 //		double hsize   = (it = attr.find("hsize")) != attr.end() ? str2double(it->second) : 612;
 //		double vsize   = (it = attr.find("vsize")) != attr.end() ? str2double(it->second) : 792;
 		double hscale  = (it = attr.find("hscale")) != attr.end() ? str2double(it->second) : 100;
@@ -326,7 +352,7 @@ void PsSpecialHandler::psfile (const string &fname, const map<string,string> &at
 }
 
 
-/** Apply transformation to width, height, and depth set by preview package. 
+/** Apply transformation to width, height, and depth set by preview package.
  *  @param[in] matrix transformation matrix to apply
  *  @param[out] w width
  *  @param[out] h height
@@ -370,8 +396,13 @@ void PsSpecialHandler::dviEndPage (unsigned) {
 			_actions->bbox().lock();
 		}
 		Message::mstream() << "\napplying bounding box set by preview package (version " << _previewFilter.version() << ")\n";
-		if (horiz_baseline)
-			Message::mstream() << "width=" << XMLString(w) << "pt, " "height=" << XMLString(h) << "pt, " "depth=" << XMLString(d) << "pt\n";
+		if (horiz_baseline) {
+			const double bp2pt = 72.27/72.0;
+			Message::mstream() <<
+				"width=" << XMLString(w*bp2pt) << "pt, "
+				"height=" << XMLString(h*bp2pt) << "pt, "
+				"depth=" << XMLString(d*bp2pt) << "pt\n";
+		}
 		else
 			Message::mstream() << "can't determine height, width, and depth due to non-horizontal baseline\n";
 	}
@@ -432,83 +463,80 @@ void PsSpecialHandler::closepath (vector<double> &p) {
 /** Draws the current path recorded by previously executed path commands (moveto, lineto,...).
  *  @param[in] p not used */
 void PsSpecialHandler::stroke (vector<double> &p) {
-	if (!_path.empty() && _actions) {
-		BoundingBox bbox;
-		if (!_actions->getMatrix().isIdentity()) {
-			_path.transform(_actions->getMatrix());
-			if (!_xmlnode)
-				bbox.transform(_actions->getMatrix());
-		}
+	if ((_path.empty() && !_clipStack.clippathLoaded()) || !_actions)
+		return;
 
-		const double pt = 72.27/72.0;  // factor to convert bp -> pt
-		ScalingMatrix scale(pt, pt);
-		_path.transform(scale);
-		bbox.transform(scale);
-
-		XMLElementNode *path=0;
-		Pair<double> point;
-		if (_path.isDot(point)) {  // zero-length path?
-			if (_linecap == 1) {    // round line ends?  => draw dot
-				double x = point.x();
-				double y = point.y();
-				double r = _linewidth/2.0;
-				path = new XMLElementNode("circle");
-				path->addAttribute("cx", x);
-				path->addAttribute("cy", y);
-				path->addAttribute("r", r);
-				path->addAttribute("fill", _actions->getColor().rgbString());
-				bbox = BoundingBox(x-r, y-r, x+r, y+r);
-			}
-		}
-		else {
-			// compute bounding box
-			_path.computeBBox(bbox);
-			bbox.expand(_linewidth/2);
-
-			ostringstream oss;
-			_path.writeSVG(oss);
-			path = new XMLElementNode("path");
-			path->addAttribute("d", oss.str());
-			path->addAttribute("stroke", _actions->getColor().rgbString());
-			path->addAttribute("fill", "none");
-			if (_linewidth != 1)
-				path->addAttribute("stroke-width", _linewidth);
-			if (_miterlimit != 4)
-				path->addAttribute("stroke-miterlimit", _miterlimit);
-			if (_linecap > 0)     // default value is "butt", no need to set it explicitly
-				path->addAttribute("stroke-linecap", _linecap == 1 ? "round" : "square");
-			if (_linejoin > 0)    // default value is "miter", no need to set it explicitly
-				path->addAttribute("stroke-linejoin", _linecap == 1 ? "round" : "bevel");
-			if (_opacityalpha < 1)
-				path->addAttribute("stroke-opacity", _opacityalpha);
-			if (!_dashpattern.empty()) {
-				ostringstream oss;
-				for (size_t i=0; i < _dashpattern.size(); i++) {
-					if (i > 0)
-						oss << ',';
-					oss << XMLString(_dashpattern[i]);
-				}
-				path->addAttribute("stroke-dasharray", oss.str());
-				if (_dashoffset != 0)
-					path->addAttribute("stroke-dashoffset", _dashoffset);
-			}
-		}
-		if (path && _clipStack.top()) {
-			// assign clipping path and clip bounding box
-			path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
-			BoundingBox clipbox;
-			_clipStack.top()->computeBBox(clipbox);
-			bbox.intersect(clipbox);
-		}
-
-		if (_xmlnode)
-			_xmlnode->append(path);
-		else {
-			_actions->appendToPage(path);
-			_actions->embed(bbox);
-		}
-		_path.newpath();
+	BoundingBox bbox;
+	if (!_actions->getMatrix().isIdentity()) {
+		_path.transform(_actions->getMatrix());
+		if (!_xmlnode)
+			bbox.transform(_actions->getMatrix());
 	}
+	if (_clipStack.clippathLoaded() && _clipStack.top())
+		_path.prepend(*_clipStack.top());
+	XMLElementNode *path=0;
+	Pair<double> point;
+	if (_path.isDot(point)) {  // zero-length path?
+		if (_linecap == 1) {    // round line ends?  => draw dot
+			double x = point.x();
+			double y = point.y();
+			double r = _linewidth/2.0;
+			path = new XMLElementNode("circle");
+			path->addAttribute("cx", x);
+			path->addAttribute("cy", y);
+			path->addAttribute("r", r);
+			path->addAttribute("fill", _actions->getColor().svgColorString());
+			bbox = BoundingBox(x-r, y-r, x+r, y+r);
+		}
+	}
+	else {
+		// compute bounding box
+		_path.computeBBox(bbox);
+		bbox.expand(_linewidth/2);
+
+		ostringstream oss;
+		_path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+		path = new XMLElementNode("path");
+		path->addAttribute("d", oss.str());
+		path->addAttribute("stroke", _actions->getColor().svgColorString());
+		path->addAttribute("fill", "none");
+		if (_linewidth != 1)
+			path->addAttribute("stroke-width", _linewidth);
+		if (_miterlimit != 4)
+			path->addAttribute("stroke-miterlimit", _miterlimit);
+		if (_linecap > 0)     // default value is "butt", no need to set it explicitly
+			path->addAttribute("stroke-linecap", _linecap == 1 ? "round" : "square");
+		if (_linejoin > 0)    // default value is "miter", no need to set it explicitly
+			path->addAttribute("stroke-linejoin", _linecap == 1 ? "round" : "bevel");
+		if (_opacityalpha < 1)
+			path->addAttribute("stroke-opacity", _opacityalpha);
+		if (!_dashpattern.empty()) {
+			ostringstream oss;
+			for (size_t i=0; i < _dashpattern.size(); i++) {
+				if (i > 0)
+					oss << ',';
+				oss << XMLString(_dashpattern[i]);
+			}
+			path->addAttribute("stroke-dasharray", oss.str());
+			if (_dashoffset != 0)
+				path->addAttribute("stroke-dashoffset", _dashoffset);
+		}
+	}
+	if (path && _clipStack.top()) {
+		// assign clipping path and clip bounding box
+		path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
+		BoundingBox clipbox;
+		_clipStack.top()->computeBBox(clipbox);
+		bbox.intersect(clipbox);
+		_clipStack.setClippathLoaded(false);
+	}
+	if (_xmlnode)
+		_xmlnode->append(path);
+	else {
+		_actions->appendToPage(path);
+		_actions->embed(bbox);
+	}
+	_path.clear();
 }
 
 
@@ -516,47 +544,47 @@ void PsSpecialHandler::stroke (vector<double> &p) {
  *  @param[in] p not used
  *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
 void PsSpecialHandler::fill (vector<double> &p, bool evenodd) {
-	if (!_path.empty() && _actions) {
-		// compute bounding box
-		BoundingBox bbox;
-		_path.computeBBox(bbox);
-		if (!_actions->getMatrix().isIdentity()) {
-			_path.transform(_actions->getMatrix());
-			if (!_xmlnode)
-				bbox.transform(_actions->getMatrix());
-		}
-		const double pt = 72.27/72.0;  // factor to convert bp -> pt
-		ScalingMatrix scale(pt, pt);
-		_path.transform(scale);
-		bbox.transform(scale);
+	if ((_path.empty() && !_clipStack.clippathLoaded()) || !_actions)
+		return;
 
-		ostringstream oss;
-		_path.writeSVG(oss);
-		XMLElementNode *path = new XMLElementNode("path");
-		path->addAttribute("d", oss.str());
-		if (_pattern)
-			path->addAttribute("fill", XMLString("url(#")+_pattern->svgID()+")");
-		else if (_actions->getColor() != Color::BLACK || _savenode)
-			path->addAttribute("fill", _actions->getColor().rgbString());
-		if (_clipStack.top()) {
-			// assign clipping path and clip bounding box
-			path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
-			BoundingBox clipbox;
-			_clipStack.top()->computeBBox(clipbox);
-			bbox.intersect(clipbox);
-		}
-		if (evenodd)  // SVG default fill rule is "nonzero" algorithm
-			path->addAttribute("fill-rule", "evenodd");
-		if (_opacityalpha < 1)
-			path->addAttribute("fill-opacity", _opacityalpha);
-		if (_xmlnode)
-			_xmlnode->append(path);
-		else {
-			_actions->appendToPage(path);
-			_actions->embed(bbox);
-		}
-		_path.newpath();
+	// compute bounding box
+	BoundingBox bbox;
+	_path.computeBBox(bbox);
+	if (!_actions->getMatrix().isIdentity()) {
+		_path.transform(_actions->getMatrix());
+		if (!_xmlnode)
+			bbox.transform(_actions->getMatrix());
 	}
+	if (_clipStack.clippathLoaded() && _clipStack.top())
+		_path.prepend(*_clipStack.top());
+
+	ostringstream oss;
+	_path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+	XMLElementNode *path = new XMLElementNode("path");
+	path->addAttribute("d", oss.str());
+	if (_pattern)
+		path->addAttribute("fill", XMLString("url(#")+_pattern->svgID()+")");
+	else if (_actions->getColor() != Color::BLACK || _savenode)
+		path->addAttribute("fill", _actions->getColor().svgColorString());
+	if (_clipStack.top()) {
+		// assign clipping path and clip bounding box
+		path->addAttribute("clip-path", XMLString("url(#clip")+XMLString(_clipStack.topID())+")");
+		BoundingBox clipbox;
+		_clipStack.top()->computeBBox(clipbox);
+		bbox.intersect(clipbox);
+		_clipStack.setClippathLoaded(false);
+	}
+	if (evenodd)  // SVG default fill rule is "nonzero" algorithm
+		path->addAttribute("fill-rule", "evenodd");
+	if (_opacityalpha < 1)
+		path->addAttribute("fill-opacity", _opacityalpha);
+	if (_xmlnode)
+		_xmlnode->append(path);
+	else {
+		_actions->appendToPage(path);
+		_actions->embed(bbox);
+	}
+	_path.clear();
 }
 
 
@@ -639,7 +667,7 @@ void PsSpecialHandler::setpattern (vector<double> &p) {
 	int pattern_id = p[0];
 	Color color;
 	if (p.size() == 4)
-		color.set((float)p[1], (float)p[2], (float)p[3]);
+		color.setRGB(p[1], p[2], p[3]);
 	map<int,PSPattern*>::iterator it = _patterns.find(pattern_id);
 	if (it == _patterns.end())
 		_pattern = 0;
@@ -657,50 +685,309 @@ void PsSpecialHandler::setpattern (vector<double> &p) {
 
 /** Clears the current clipping path.
  *  @param[in] p not used */
-void PsSpecialHandler::initclip (vector<double> &p) {
-	_clipStack.push();  // push empty path
+void PsSpecialHandler::initclip (vector<double> &) {
+	_clipStack.pushEmptyPath();
 }
 
 
-/** Assigns a new clipping path.
+/** Assigns the current clipping path to the graphics path. */
+void PsSpecialHandler::clippath (std::vector<double>&) {
+	if (!_clipStack.empty()) {
+		_clipStack.setClippathLoaded(true);
+		_path.clear();
+	}
+}
+
+
+/** Assigns a new clipping path to the graphics state using the current path.
+ *  If the graphics state already contains a clipping path, the new one is
+ *  computed by intersecting the current clipping path with the current graphics
+ *  path (see PS language reference, 3rd edition, pp. 193, 542)
  *  @param[in] p not used
  *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
-void PsSpecialHandler::clip (vector<double> &p, bool evenodd) {
-	// when this method is called, _path contains the clipping path
-	if (!_path.empty() && _actions) {
-		if (!_actions->getMatrix().isIdentity())
-			_path.transform(_actions->getMatrix());
-
-		const double pt = 72.27/72.0;  // factor to convert bp -> pt
-		ScalingMatrix scale(pt, pt);
-		_path.transform(scale);
+void PsSpecialHandler::clip (vector<double> &, bool evenodd) {
+	clip(_path, evenodd);
+}
 
 
-		int oldID = _clipStack.topID();
-		_clipStack.replace(_path);
-		int newID = _clipStack.topID();
+/** Assigns a new clipping path to the graphics state using the current path.
+ *  If the graphics state already contains a clipping path, the new one is
+ *  computed by intersecting the current one with the given path.
+ *  @param[in] path path used to restrict the clipping region
+ *  @param[in] evenodd true: use even-odd fill algorithm, false: use nonzero fill algorithm */
+void PsSpecialHandler::clip (Path &path, bool evenodd) {
+		// when this method is called, _path contains the clipping path
+	if (path.empty() || !_actions)
+		return;
 
-		ostringstream oss;
-		_path.writeSVG(oss);
-		XMLElementNode *path = new XMLElementNode("path");
-		path->addAttribute("d", oss.str());
-		if (evenodd)
-			path->addAttribute("clip-rule", "evenodd");
+	Path::WindingRule windingRule = evenodd ? Path::WR_EVEN_ODD : Path::WR_NON_ZERO;
+	path.setWindingRule(windingRule);
 
-		XMLElementNode *clip = new XMLElementNode("clipPath");
-		clip->addAttribute("id", XMLString("clip")+XMLString(newID));
-		if (oldID)
-			clip->addAttribute("clip-path", XMLString("url(#clip")+XMLString(oldID)+")");
+	if (!_actions->getMatrix().isIdentity())
+		path.transform(_actions->getMatrix());
 
-		clip->append(path);
-		_actions->appendToDefs(clip);
+	int oldID = _clipStack.topID();
+
+	ostringstream oss;
+	if (!COMPUTE_CLIPPATHS_INTERSECTIONS || oldID < 1) {
+		_clipStack.replace(path);
+		path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+	}
+	else {
+		// compute the intersection of the current clipping path with the current graphics path
+		Path *oldPath = _clipStack.getPath(oldID);
+		Path intersectedPath(windingRule);
+		PathClipper clipper;
+		clipper.intersect(*oldPath, path, intersectedPath);
+		_clipStack.replace(intersectedPath);
+		intersectedPath.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+	}
+
+	XMLElementNode *pathElem = new XMLElementNode("path");
+	pathElem->addAttribute("d", oss.str());
+	if (evenodd)
+		pathElem->addAttribute("clip-rule", "evenodd");
+
+	int newID = _clipStack.topID();
+	XMLElementNode *clipElem = new XMLElementNode("clipPath");
+	clipElem->addAttribute("id", XMLString("clip")+XMLString(newID));
+	if (!COMPUTE_CLIPPATHS_INTERSECTIONS && oldID)
+		clipElem->addAttribute("clip-path", XMLString("url(#clip")+XMLString(oldID)+")");
+
+	clipElem->append(pathElem);
+	_actions->appendToDefs(clipElem);
+}
+
+
+/** Applies a gradient fill to the current graphics path. Vector p contains the shading parameters
+ *  in the following order:
+ *  - shading type (6=Coons, 7=tensor product)
+ *  - color space (1=gray, 3=rgb, 4=cmyk)
+ *  - 1.0 followed by the background color components based on the declared color space, or 0.0
+ *  - 1.0 followed by the bounding box coordinates, or 0.0
+ *  - geometry and color parameters depending on the shading type */
+void PsSpecialHandler::shfill (vector<double> &params) {
+	if (params.size() < 9)
+		return;
+
+	// collect common data relevant for all shading types
+	int shadingTypeID = static_cast<int>(params[0]);
+	ColorSpace colorSpace = Color::RGB_SPACE;
+	switch (static_cast<int>(params[1])) {
+		case 1: colorSpace = Color::GRAY_SPACE; break;
+		case 3: colorSpace = Color::RGB_SPACE; break;
+		case 4: colorSpace = Color::CMYK_SPACE; break;
+	}
+	VectorIterator<double> it = params;
+	it += 2;     // skip shading type and color space
+	// Get color to fill the whole mesh area before drawing the gradient colors on top of that background.
+	// This is an optional parameter to shfill.
+	bool bgcolorGiven = static_cast<bool>(*it++);
+	Color bgcolor;
+	if (bgcolorGiven)
+		bgcolor.set(colorSpace, it);
+	// Get clipping rectangle to limit the drawing area of the gradient mesh.
+	// This is an optional parameter to shfill too.
+	bool bboxGiven = static_cast<bool>(*it++);
+	if (bboxGiven) { // bounding box given
+		Path bboxPath;
+		const double &x1 = *it++;
+		const double &y1 = *it++;
+		const double &x2 = *it++;
+		const double &y2 = *it++;
+		bboxPath.moveto(x1, y1);
+		bboxPath.lineto(x2, y1);
+		bboxPath.lineto(x2, y2);
+		bboxPath.lineto(x1, y2);
+		bboxPath.closepath();
+		clip(bboxPath, false);
+	}
+	try {
+		if (shadingTypeID == 5)
+			processLatticeTriangularPatchMesh(colorSpace, it);
+		else
+			processSequentialPatchMesh(shadingTypeID, colorSpace, it);
+	}
+	catch (ShadingException &e) {
+		Message::estream(false) << "PostScript error: " << e.what() << '\n';
+		it.invalidate();  // stop processing the remaining patch data
+	}
+	catch (IteratorException &e) {
+		Message::estream(false) << "PostScript error: incomplete shading data\n";
+	}
+	if (bboxGiven)
+		_clipStack.pop();
+}
+
+
+/** Reads position and color data of a single shading patch from the data vector.
+ *  @param[in] shadingTypeID PS shading type ID identifying the format of the subsequent patch data
+ *  @param[in] edgeflag edge flag specifying how to connect the current patch to the preceding one
+ *  @param[in] cspace color space used to compute the color gradient
+ *  @param[in,out] it iterator used to sequentially access the patch data
+ *  @param[out] points the points defining the geometry of the patch
+ *  @param[out] colors the colors assigned to the vertices of the patch */
+static void read_patch_data (ShadingPatch &patch, int edgeflag,
+		VectorIterator<double> &it, vector<DPair> &points, vector<Color> &colors)
+{
+	// number of control points and colors required to define a single patch
+	int numPoints = patch.numPoints(edgeflag);
+	int numColors = patch.numColors(edgeflag);
+	points.resize(numPoints);
+	colors.resize(numColors);
+	if (patch.psShadingType() == 4) {
+		// format of a free-form triangular patch definition, where eN denotes
+		// the edge of the corresponding vertex:
+		// edge flag = 0, x1, y1, {color1}, e2, x2, y2, {color2}, e3, x3, y3, {color3}
+		// edge flag > 0, x1, y1, {color1}
+		for (int i=0; i < numPoints; i++) {
+			if (i > 0) ++it;  // skip redundant edge flag from free-form triangular patch
+			double x = *it++;
+			double y = *it++;
+			points[i] = DPair(x, y);
+			colors[i].set(patch.colorSpace(), it);
+		}
+	}
+	else if (patch.psShadingType() == 6 || patch.psShadingType() == 7) {
+		// format of each Coons/tensor product patch definition:
+		// edge flag = 0, x1, y1, ... , xn, yn, {color1}, {color2}, {color3}, {color4}
+		// edge flag > 0, x5, y5, ... , xn, yn, {color3}, {color4}
+		for (int i=0; i < numPoints; i++) {
+			double x = *it++;
+			double y = *it++;
+			points[i] = DPair(x, y);
+		}
+		for (int i=0; i < numColors; i++)
+			colors[i].set(patch.colorSpace(), it);
+	}
+}
+
+
+class ShadingCallback : public ShadingPatch::Callback {
+	public:
+		ShadingCallback (SpecialActions *actions, XMLElementNode *parent, int clippathID)
+			: _actions(actions), _group(new XMLElementNode("g"))
+		{
+			if (parent)
+				parent->append(_group);
+			else
+				actions->appendToPage(_group);
+			if (clippathID > 0)
+				_group->addAttribute("clip-path", XMLString("url(#clip")+XMLString(clippathID)+")");
+		}
+
+		void patchSegment (GraphicPath<double> &path, const Color &color) {
+			if (!_actions->getMatrix().isIdentity())
+				path.transform(_actions->getMatrix());
+
+			// draw a single patch segment
+			ostringstream oss;
+			path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+			XMLElementNode *pathElem = new XMLElementNode("path");
+			pathElem->addAttribute("d", oss.str());
+			pathElem->addAttribute("fill", color.svgColorString());
+			_group->append(pathElem);
+		}
+
+	private:
+		SpecialActions *_actions;
+		XMLElementNode *_group;
+};
+
+
+/** Handle all patch meshes whose patches and their connections can be processed sequentially.
+ *  This comprises free-form triangular, Coons, and tensor-product patch meshes. */
+void PsSpecialHandler::processSequentialPatchMesh (int shadingTypeID, ColorSpace colorSpace, VectorIterator<double> &it) {
+	auto_ptr<ShadingPatch> previousPatch;
+	while (it.valid()) {
+		int edgeflag = static_cast<int>(*it++);
+		vector<DPair> points;
+		vector<Color> colors;
+		auto_ptr<ShadingPatch> patch;
+
+		patch = auto_ptr<ShadingPatch>(ShadingPatch::create(shadingTypeID, colorSpace));
+		read_patch_data(*patch, edgeflag, it, points, colors);
+		patch->setPoints(points, edgeflag, previousPatch.get());
+		patch->setColors(colors, edgeflag, previousPatch.get());
+		ShadingCallback callback(_actions, _xmlnode, _clipStack.topID());
+#if 0
+		if (bgcolorGiven) {
+			// fill whole patch area with given background color
+			GraphicPath<double> outline;
+			patch->getBoundaryPath(outline);
+			callback.patchSegment(outline, bgcolor);
+		}
+#endif
+		patch->approximate(SHADING_SEGMENT_SIZE, SHADING_SEGMENT_OVERLAP, SHADING_SIMPLIFY_DELTA, callback);
+		if (!_xmlnode) {
+			// update bounding box
+			BoundingBox bbox;
+			patch->getBBox(bbox);
+			bbox.transform(_actions->getMatrix());
+			_actions->embed(bbox);
+		}
+		previousPatch = patch;
+	}
+}
+
+
+struct PatchVertex {
+	DPair point;
+	Color color;
+};
+
+
+void PsSpecialHandler::processLatticeTriangularPatchMesh (ColorSpace colorSpace, VectorIterator<double> &it) {
+	int verticesPerRow = static_cast<int>(*it++);
+	if (verticesPerRow < 2)
+		return;
+
+	// hold two adjacent rows of vertices and colors
+	vector<PatchVertex> row1(verticesPerRow);
+	vector<PatchVertex> row2(verticesPerRow);
+	vector<PatchVertex> *rowptr1 = &row1;
+	vector<PatchVertex> *rowptr2 = &row2;
+	// read data of first row
+	for (int i=0; i < verticesPerRow; i++) {
+		PatchVertex &vertex = (*rowptr1)[i];
+		vertex.point.x(*it++);
+		vertex.point.y(*it++);
+		vertex.color.set(colorSpace, it);
+	}
+	LatticeTriangularPatch patch(colorSpace);
+	ShadingCallback callback(_actions, _xmlnode, _clipStack.topID());
+	while (it.valid()) {
+		// read next row
+		for (int i=0; i < verticesPerRow; i++) {
+			PatchVertex &vertex = (*rowptr2)[i];
+			vertex.point.x(*it++);
+			vertex.point.y(*it++);
+			vertex.color.set(colorSpace, it);
+		}
+		// create triangular patches for the vertices of the two rows
+		for (int i=0; i < verticesPerRow-1; i++) {
+			const PatchVertex &v1 = (*rowptr1)[i], &v2 = (*rowptr1)[i+1];
+			const PatchVertex &v3 = (*rowptr2)[i], &v4 = (*rowptr2)[i+1];
+			patch.setPoints(v1.point, v2.point, v3.point);
+			patch.setColors(v1.color, v2.color, v3.color);
+			patch.approximate(SHADING_SEGMENT_SIZE, SHADING_SEGMENT_OVERLAP, SHADING_SIMPLIFY_DELTA, callback);
+
+			patch.setPoints(v2.point, v3.point, v4.point);
+			patch.setColors(v2.color, v3.color, v4.color);
+			patch.approximate(SHADING_SEGMENT_SIZE, SHADING_SEGMENT_OVERLAP, SHADING_SIMPLIFY_DELTA, callback);
+		}
+		swap(rowptr1, rowptr2);
 	}
 }
 
 
 /** Clears current path */
 void PsSpecialHandler::newpath (vector<double> &p) {
-	_path.newpath();
+	bool drawing = (p[0] > 0);
+	if (!drawing || !_clipStack.clippathLoaded()) {
+		_path.clear();
+		_clipStack.setClippathLoaded(false);
+	}
 }
 
 
@@ -750,38 +1037,33 @@ void PsSpecialHandler::rotate (vector<double> &p) {
 
 void PsSpecialHandler::setgray (vector<double> &p) {
 	_pattern = 0;
-	if (_actions) {
-		Color c;
-		c.setGray((float)p[0]);
-		_actions->setColor(c);
-	}
+	_currentcolor.setGray(p[0]);
+	if (_actions)
+		_actions->setColor(_currentcolor);
 }
 
 
 void PsSpecialHandler::setrgbcolor (vector<double> &p) {
 	_pattern= 0;
+	_currentcolor.setRGB(p[0], p[1], p[2]);
 	if (_actions)
-		_actions->setColor(Color((float)p[0], (float)p[1], (float)p[2]));
+		_actions->setColor(_currentcolor);
 }
 
 
 void PsSpecialHandler::setcmykcolor (vector<double> &p) {
 	_pattern = 0;
-	if (_actions) {
-		Color c;
-		c.setCMYK((float)p[0], (float)p[1], (float)p[2], (float)p[3]);
-		_actions->setColor(c);
-	}
+	_currentcolor.setCMYK(p[0], p[1], p[2], p[3]);
+	if (_actions)
+		_actions->setColor(_currentcolor);
 }
 
 
 void PsSpecialHandler::sethsbcolor (vector<double> &p) {
 	_pattern = 0;
-	if (_actions) {
-		Color c;
-		c.setHSB((float)p[0], (float)p[1], (float)p[2]);
-		_actions->setColor(c);
-	}
+	_currentcolor.setHSB(p[0], p[1], p[2]);
+	if (_actions)
+		_actions->setColor(_currentcolor);
 }
 
 
@@ -803,7 +1085,7 @@ void PsSpecialHandler::executed () {
 
 ////////////////////////////////////////////
 
-void PsSpecialHandler::ClippingStack::push () {
+void PsSpecialHandler::ClippingStack::pushEmptyPath () {
 	if (!_stack.empty())
 		_stack.push(Entry(0, -1));
 }
@@ -857,6 +1139,24 @@ const PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::top () const {
 }
 
 
+PsSpecialHandler::Path* PsSpecialHandler::ClippingStack::getPath (size_t id) {
+	return (id > 0 && id <= _paths.size()) ? &_paths[id-1] : 0;
+}
+
+
+/** Returns true if the clipping path was loaded into the graphics path (via PS operator 'clippath') */
+bool PsSpecialHandler::ClippingStack::clippathLoaded () const {
+	return !_stack.empty() && _stack.top().cpathLoaded;
+}
+
+
+void PsSpecialHandler::ClippingStack::setClippathLoaded (bool loaded) {
+	if (_stack.empty())
+		return;
+	_stack.top().cpathLoaded = loaded;
+}
+
+
 /** Pops all elements from the stack. */
 void PsSpecialHandler::ClippingStack::clear() {
 	_paths.clear();
@@ -888,4 +1188,3 @@ const char** PsSpecialHandler::prefixes () const {
 	static const char *pfx[] = {"header=", "psfile=", "PSfile=", "ps:", "ps::", "!", "\"", 0};
 	return pfx;
 }
-
