@@ -2,7 +2,7 @@
 ** SVGTree.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -23,6 +23,7 @@
 #include <sstream>
 #include <string>
 #include "BoundingBox.h"
+#include "DependencyGraph.h"
 #include "DVIToSVG.h"
 #include "Font.h"
 #include "FontManager.h"
@@ -38,6 +39,9 @@ using namespace std;
 bool SVGTree::CREATE_STYLE=true;
 bool SVGTree::USE_FONTS=true;
 bool SVGTree::CREATE_USE_ELEMENTS=false;
+bool SVGTree::RELATIVE_PATH_CMDS=false;
+bool SVGTree::MERGE_CHARS=true;
+double SVGTree::ZOOM_FACTOR=1.0;
 
 
 SVGTree::SVGTree () : _vertical(false), _font(0), _color(Color::BLACK), _matrix(1) {
@@ -60,15 +64,13 @@ void SVGTree::reset () {
 
 
 /** Sets the bounding box of the document.
- *  @param[in] bbox bounding box in TeX point units */
+ *  @param[in] bbox bounding box in PS point units */
 void SVGTree::setBBox (const BoundingBox &bbox) {
-	double bp = 72.0/72.27;  // pt -> bp
-	ScalingMatrix scale(bp, bp);
-	BoundingBox scaled_box = bbox;
-	scaled_box.transform(scale);
-	_root->addAttribute("width", XMLString(scaled_box.width())+"pt");
-	_root->addAttribute("height", XMLString(scaled_box.height())+"pt");
-	_root->addAttribute("viewBox", scaled_box.toSVGViewBox());
+	if (ZOOM_FACTOR >= 0) {
+		_root->addAttribute("width", XMLString(bbox.width()*ZOOM_FACTOR)+"pt");
+		_root->addAttribute("height", XMLString(bbox.height()*ZOOM_FACTOR)+"pt");
+	}
+	_root->addAttribute("viewBox", bbox.toSVGViewBox());
 }
 
 
@@ -114,6 +116,8 @@ void SVGTree::appendToPage (XMLNode *node) {
 		_page->append(node);
 	else
 		_pageContainerStack.top()->append(node);
+	if (node != _text) // if the appended node differ from text element currently in use,
+		_text = 0;      // then force creating a new text element for the following characters
 }
 
 
@@ -135,12 +139,12 @@ void SVGTree::appendChar (int c, double x, double y, const Font &font) {
 	XMLElementNode *node=_span;
 	if (USE_FONTS) {
 		// changes of fonts and transformations require a new text element
-		if (!_text || _font.changed() || _matrix.changed() || _vertical.changed()) {
+		if (!MERGE_CHARS || !_text || _font.changed() || _matrix.changed() || _vertical.changed()) {
 			newTextNode(x, y);
 			node = _text;
 			_color.changed(true);
 		}
-		if (_xchanged || _ychanged || (_color.changed() && _color.get() != Color::BLACK)) {
+		if (MERGE_CHARS && (_xchanged || _ychanged || (_color.changed() && _color.get() != Color::BLACK))) {
 			// if drawing position was explicitly changed, create a new tspan element
 			_span = new XMLElementNode("tspan");
 			if (_xchanged) {
@@ -158,7 +162,7 @@ void SVGTree::appendChar (int c, double x, double y, const Font &font) {
 				_ychanged = false;
 			}
 			if (_color.get() != font.color()) {
-					_span->addAttribute("fill", _color.get().rgbString());
+					_span->addAttribute("fill", _color.get().svgColorString());
 				_color.changed(false);
 			}
 			_text->append(_span);
@@ -170,6 +174,10 @@ void SVGTree::appendChar (int c, double x, double y, const Font &font) {
 			node = _text;
 		}
 		node->append(XMLString(font.unicode(c), false));
+		if (!MERGE_CHARS && _color.get() != font.color()) {
+			node->addAttribute("fill", _color.get().svgColorString());
+			_color.changed(false);
+		}
 	}
 	else {
 		if (_color.changed() || _matrix.changed()) {
@@ -178,7 +186,7 @@ void SVGTree::appendChar (int c, double x, double y, const Font &font) {
 			if (set_color || set_matrix) {
 				_span = new XMLElementNode("g");
 				if (_color.get() != Color::BLACK)
-					_span->addAttribute("fill", _color.get().rgbString());
+					_span->addAttribute("fill", _color.get().svgColorString());
 				if (!_matrix.get().isIdentity())
 					_span->addAttribute("transform", _matrix.get().getSVG());
 				appendToPage(_span);
@@ -233,7 +241,7 @@ void SVGTree::appendChar (int c, double x, double y, const Font &font) {
 				double sx = pf->scaledSize()/pf->unitsPerEm();
 				double sy = -sx;
 				ostringstream oss;
-				glyph.writeSVG(oss, sx, sy, x, y);
+				glyph.writeSVG(oss, RELATIVE_PATH_CMDS, sx, sy, x, y);
 				XMLElementNode *glyph_node = new XMLElementNode("path");
 				glyph_node->addAttribute("d", oss.str());
 				if (!rotation.isIdentity())
@@ -257,9 +265,9 @@ void SVGTree::newTextNode (double x, double y) {
 			_text->addAttribute("class", string("f")+XMLString(_fontnum));
 		else {
 			_text->addAttribute("font-family", font->name());
-			_text->addAttribute("font-size", font->scaledSize());
+			_text->addAttribute("font-size", XMLString(font->scaledSize()));
 			if (font->color() != Color::BLACK)
-				_text->addAttribute("fill", font->color().rgbString());
+				_text->addAttribute("fill", font->color().svgColorString());
 		}
 		if (_vertical) {
 			_text->addAttribute("writing-mode", "tb");
@@ -285,18 +293,15 @@ void SVGTree::newTextNode (double x, double y) {
 
 
 void SVGTree::transformPage (const Matrix *usermatrix) {
-	double bp = 72.0/72.27;  // pt -> bp
-	ScalingMatrix matrix(bp, bp);
-	if (usermatrix)
-		matrix.rmultiply(*usermatrix);
-	if (!matrix.isIdentity())
-		_page->addAttribute("transform", matrix.getSVG());
+	if (usermatrix && !usermatrix->isIdentity())
+		_page->addAttribute("transform", usermatrix->getSVG());
 }
 
 
 /** Creates an SVG element for a single glyph.
  *  @param[in] c character number
  *  @param[in] font font to extract the glyph from
+ *  @param[in] cb pointer to callback object for sending feedback to the glyph tracer (may be 0)
  *  @return pointer to element node if glyph exists, 0 otherwise */
 static XMLElementNode* createGlyphNode (int c, const PhysicalFont &font, GFGlyphTracer::Callback *cb) {
 	Glyph glyph;
@@ -325,7 +330,7 @@ static XMLElementNode* createGlyphNode (int c, const PhysicalFont &font, GFGlyph
 		sy = -sx;
 	}
 	ostringstream oss;
-	glyph.writeSVG(oss, sx, sy);
+	glyph.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS, sx, sy);
 	glyph_node->addAttribute("d", oss.str());
 	return glyph_node;
 }
@@ -346,9 +351,9 @@ void SVGTree::appendFontStyles (const set<const Font*> &fonts) {
 		FORALL(sortmap, SortMap::const_iterator, it) {
 			style << "text.f"     << it->first << ' '
 				<< "{font-family:" << it->second->name()
-				<< ";font-size:"   << it->second->scaledSize() << "px";
+				<< ";font-size:"   << XMLString(it->second->scaledSize()) << "px";
 			if (it->second->color() != Color::BLACK)
-				style << ";fill:" << it->second->color().rgbString();
+				style << ";fill:" << it->second->color().svgColorString();
 			style << "}\n";
 		}
 		XMLCDataNode *cdata = new XMLCDataNode(style.str());
@@ -359,7 +364,8 @@ void SVGTree::appendFontStyles (const set<const Font*> &fonts) {
 
 /** Appends glyph definitions of a given font to the defs section of the SVG tree.
  *  @param[in] font font to be appended
- *  @param[in] chars codes of the characters whose glyph outlines should be appended */
+ *  @param[in] chars codes of the characters whose glyph outlines should be appended
+ *  @param[in] cb pointer to callback object for sending feedback to the glyph tracer (may be 0) */
 void SVGTree::append (const PhysicalFont &font, const set<int> &chars, GFGlyphTracer::Callback *cb) {
 	if (chars.empty())
 		return;
@@ -379,11 +385,6 @@ void SVGTree::append (const PhysicalFont &font, const set<int> &chars, GFGlyphTr
 			faceNode->addAttribute("descent", XMLString(font.descent()));
 		}
 		fontNode->append(faceNode);
-
-		// add required missing-glyph element
-		XMLElementNode *missing = new XMLElementNode("missing-glyph");
-		missing->addAttribute("d", "");
-		fontNode->append(missing);
 		FORALL(chars, set<int>::const_iterator, i)
 			fontNode->append(createGlyphNode(*i, font, cb));
 	}
@@ -434,3 +435,45 @@ void SVGTree::popContextElement () {
 		_text = _span = 0; // ensure the creation of a new text element for the following characters added
 	}
 }
+
+
+/** Extracts the ID from a local URL reference like url(#abcde) */
+inline string extract_id_from_url (const string &url) {
+	return url.substr(5, url.length()-6);
+}
+
+
+/** Removes elements present in the SVH tree that are not required.
+ *  For now, only clipPath elements are removed. */
+void SVGTree::removeRedundantElements () {
+	vector<XMLElementNode*> clipElements;
+	if (!_defs || !_defs->getDescendants("clipPath", 0, clipElements))
+		return;
+
+	// collect dependencies between clipPath elements in the defs section of the SVG tree
+	DependencyGraph<string> idTree;
+	for (vector<XMLElementNode*>::iterator it=clipElements.begin(); it != clipElements.end(); ++it) {
+		if (const char *id = (*it)->getAttributeValue("id")) {
+			if (const char *url = (*it)->getAttributeValue("clip-path"))
+				idTree.insert(extract_id_from_url(url), id);
+			else
+				idTree.insert(id);
+		}
+	}
+	// collect elements that reference a clipPath (have a clip-path attribute)
+	vector<XMLElementNode*> descendants;
+	_page->getDescendants(0, "clip-path", descendants);
+	// remove referenced IDs and their dependencies from the dependency graph
+	for (vector<XMLElementNode*>::iterator it=descendants.begin(); it != descendants.end(); ++it) {
+		string idref = extract_id_from_url((*it)->getAttributeValue("clip-path"));
+		idTree.removeDependencyPath(idref);
+	}
+	descendants.clear();
+	vector<string> ids;
+	idTree.getKeys(ids);
+	for (vector<string>::iterator it=ids.begin(); it != ids.end(); ++it) {
+		XMLElementNode *node = _defs->getFirstDescendant("clipPath", "id", it->c_str());
+		_defs->remove(node);
+	}
+}
+
