@@ -2,7 +2,7 @@
 ** TpicSpecialHandler.cpp                                               **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -18,6 +18,7 @@
 ** along with this program; if not, see <http://www.gnu.org/licenses/>. **
 *************************************************************************/
 
+#define _USE_MATH_DEFINES
 #include <config.h>
 #include <cmath>
 #include <cstring>
@@ -25,7 +26,9 @@
 #include "Color.h"
 #include "InputBuffer.h"
 #include "InputReader.h"
+#include "GraphicPath.h"
 #include "SpecialActions.h"
+#include "SVGTree.h"
 #include "TpicSpecialHandler.h"
 #include "XMLNode.h"
 #include "XMLString.h"
@@ -54,8 +57,9 @@ void TpicSpecialHandler::reset () {
 /** Creates SVG elements that draw lines through the recorded points.
  *  @param[in] stroke if true, the (out)line is drawn (in black)
  *  @param[in] fill if true, enclosed area is filled with current color
- *  @param[in] ddist dash/dot distance of line in TeX point units
- *                   (0:solid line, >0:dashed line, <0:dotted line) */
+ *  @param[in] ddist dash/dot distance of line in PS point units
+ *                   (0:solid line, >0:dashed line, <0:dotted line)
+ *  @param[in] actions object providing the actions that can be performed by the SpecialHandler */
 void TpicSpecialHandler::drawLines (bool stroke, bool fill, double ddist, SpecialActions *actions) {
 	if (actions && !_points.empty()) {
 		XMLElementNode *elem=0;
@@ -81,14 +85,14 @@ void TpicSpecialHandler::drawLines (bool stroke, bool fill, double ddist, Specia
 				Color color = actions->getColor();
 				color *= _fill;
 				elem = new XMLElementNode("polygon");
-				elem->addAttribute("fill", fill ? color.rgbString() : "none");
+				elem->addAttribute("fill", fill ? color.svgColorString() : "none");
 			}
 			ostringstream oss;
 			FORALL(_points, vector<DPair>::iterator, it) {
 				if (it != _points.begin())
 					oss << ' ';
 				double x = it->x()+actions->getX();
-			  	double y = it->y()+actions->getY();
+				double y = it->y()+actions->getY();
 				oss << XMLString(x) << ',' << XMLString(y);
 				actions->embed(DPair(x, y));
 			}
@@ -114,58 +118,58 @@ void TpicSpecialHandler::drawLines (bool stroke, bool fill, double ddist, Specia
  *  with a straight line from the mid-point of the last line to the last point.
  *  If ddist=0, the spline is stroked solid. Otherwise ddist denotes the length
  *  of the dashes and the gaps inbetween.
- *  @param[in] ddist length of dashes and gaps */
+ *  @param[in] ddist length of dashes and gaps
+ *  @param[in] actions object providing the actions that can be performed by the SpecialHandler */
 void TpicSpecialHandler::drawSplines (double ddist, SpecialActions *actions) {
-	if (actions && !_points.empty()) {
-		const size_t size = _points.size();
-		if (size < 3)
-			drawLines(true, false, ddist, actions);
-		else {
-			double x = actions->getX();
-			double y = actions->getY();
-			DPair p(x,y);
-			ostringstream oss;
-			oss << 'M' << XMLString(x+_points[0].x()) << ',' << XMLString(y+_points[0].y());
-			DPair mid = p+_points[0]+(_points[1]-_points[0])/2.0;
-			oss << 'L' << XMLString(mid.x()) << ',' << XMLString(mid.y());
-			actions->embed(p+_points[0]);
-			for (size_t i=1; i < size-1; i++) {
-				const DPair p0 = p+_points[i-1];
-				const DPair p1 = p+_points[i];
-				const DPair p2 = p+_points[i+1];
-				mid = p1+(p2-p1)/2.0;
-				oss << 'Q' << XMLString(p1.x()) << ',' << XMLString(p1.y())
-					 << ' ' << XMLString(mid.x()) << ',' << XMLString(mid.y());
-				actions->embed(mid);
-				actions->embed((p0+p1*6.0+p2)/8.0, _penwidth);
-			}
-			if (_points[0] == _points[size-1])  // closed path?
-				oss << 'Z';
-			else {
-				oss << 'L' << XMLString(x+_points[size-1].x()) << ',' << XMLString(y+_points[size-1].y());
-				actions->embed(p+_points[size-1]);
-			}
-
-			Color color = actions->getColor();
-			color *= _fill;
-			XMLElementNode *path = new XMLElementNode("path");
-			if (_fill >= 0) {
-				if (_points[0] != _points[size-1])
-					oss << 'Z';
-				path->addAttribute("fill", color.rgbString());
-			}
-			else
-				path->addAttribute("fill", "none");
-
-			path->addAttribute("d", oss.str());
-			path->addAttribute("stroke", actions->getColor().rgbString());
-			path->addAttribute("stroke-width", XMLString(_penwidth));
-			if (ddist > 0)
-				path->addAttribute("stroke-dasharray", XMLString(ddist));
-			else if (ddist < 0)
-				path->addAttribute("stroke-dasharray", XMLString(_penwidth) + ' ' + XMLString(-ddist));
-			actions->appendToPage(path);
+	if (!actions || _points.empty())
+		return;
+	const size_t size = _points.size();
+	if (size < 3)
+		drawLines(true, false, ddist, actions);
+	else {
+		DPair p(actions->getX(), actions->getY());
+		GraphicPath<double> path;
+		path.moveto(p+_points[0]);
+		DPair mid = p+_points[0]+(_points[1]-_points[0])/2.0;
+		path.lineto(mid);
+		actions->embed(p+_points[0]);
+		for (size_t i=1; i < size-1; i++) {
+			const DPair p0 = p+_points[i-1];
+			const DPair p1 = p+_points[i];
+			const DPair p2 = p+_points[i+1];
+			mid = p1+(p2-p1)/2.0;
+			path.conicto(p1, mid);
+			actions->embed(mid);
+			actions->embed((p0+p1*6.0+p2)/8.0, _penwidth);
 		}
+		if (_points[0] == _points[size-1])  // closed path?
+			path.closepath();
+		else {
+			path.lineto(p+_points[size-1]);
+			actions->embed(p+_points[size-1]);
+		}
+
+		Color color = actions->getColor();
+		color *= _fill;
+		XMLElementNode *pathElem = new XMLElementNode("path");
+		if (_fill >= 0) {
+			if (_points[0] != _points[size-1])
+				path.closepath();
+			pathElem->addAttribute("fill", color.svgColorString());
+		}
+		else
+			pathElem->addAttribute("fill", "none");
+
+		ostringstream oss;
+		path.writeSVG(oss, SVGTree::RELATIVE_PATH_CMDS);
+		pathElem->addAttribute("d", oss.str());
+		pathElem->addAttribute("stroke", actions->getColor().svgColorString());
+		pathElem->addAttribute("stroke-width", XMLString(_penwidth));
+		if (ddist > 0)
+			pathElem->addAttribute("stroke-dasharray", XMLString(ddist));
+		else if (ddist < 0)
+			pathElem->addAttribute("stroke-dasharray", XMLString(_penwidth) + ' ' + XMLString(-ddist));
+		actions->appendToPage(pathElem);
 	}
 	reset();
 }
@@ -177,10 +181,11 @@ void TpicSpecialHandler::drawSplines (double ddist, SpecialActions *actions) {
  *  @param[in] rx length of horizonal semi-axis
  *  @param[in] ry length of vertical semi-axis
  *  @param[in] angle1 starting angle (clockwise) relative to x-axis
- *  @param[in] angle2 ending angle (clockwise) relative to x-axis */
+ *  @param[in] angle2 ending angle (clockwise) relative to x-axis
+ *  @param[in] actions object providing the actions that can be performed by the SpecialHandler */
 void TpicSpecialHandler::drawArc (double cx, double cy, double rx, double ry, double angle1, double angle2, SpecialActions *actions) {
 	if (actions) {
-		const double PI2 = 4*asin(1.0);
+		const double PI2 = 2*M_PI;
 		angle1 *= -1;
 		angle2 *= -1;
 		if (fabs(angle1) > PI2) {
@@ -205,7 +210,7 @@ void TpicSpecialHandler::drawArc (double cx, double cy, double rx, double ry, do
 			if (angle2 < 0)
 				angle2 = PI2+angle2;
 			elem = new XMLElementNode("path");
-			int large_arg = fabs(angle1-angle2) > PI2/2 ? 0 : 1;
+			int large_arg = fabs(angle1-angle2) > M_PI ? 0 : 1;
 			int sweep_flag = angle1 > angle2 ? 0 : 1;
 			if (angle1 > angle2) {
 				large_arg = 1-large_arg;
@@ -222,13 +227,13 @@ void TpicSpecialHandler::drawArc (double cx, double cy, double rx, double ry, do
 			elem->addAttribute("d", oss.str());
 		}
 		elem->addAttribute("stroke-width", _penwidth);
-		elem->addAttribute("stroke", actions->getColor().rgbString());
+		elem->addAttribute("stroke", actions->getColor().svgColorString());
 		elem->addAttribute("stroke-linecap", "round");
 		elem->addAttribute("fill", "none");
 		if (_fill >= 0) {
 			Color color=actions->getColor();
 			color *= _fill;
-			elem->addAttribute("fill", color.rgbString());
+			elem->addAttribute("fill", color.svgColorString());
 		}
 		else
 			elem->addAttribute("fill", "none");
@@ -245,12 +250,12 @@ bool TpicSpecialHandler::process (const char *prefix, istream &is, SpecialAction
 	if (!prefix || strlen(prefix) != 2)
 		return false;
 
-	const double PT=0.07227; // factor for milli-inch to TeX points
+	const double mi2bp=0.072; // factor for milli-inch to PS points
 	StreamInputBuffer ib(is);
 	BufferInputReader in(ib);
 	switch (cmd_id(prefix[0], prefix[1])) {
 		case cmd_id('p','n'): // set pen width in milli-inches
-			_penwidth = in.getDouble()*PT;
+			_penwidth = in.getDouble()*mi2bp;
 			break;
 		case cmd_id('b','k'): // set fill color to black
 			_fill = 0;
@@ -265,8 +270,8 @@ bool TpicSpecialHandler::process (const char *prefix, istream &is, SpecialAction
 		case cmd_id('t','x'): // set fill pattern
 			break;
 		case cmd_id('p','a'): { // add point to path
-			double x=in.getDouble()*PT;
-			double y=in.getDouble()*PT;
+			double x = in.getDouble()*mi2bp;
+			double y = in.getDouble()*mi2bp;
 			_points.push_back(DPair(x,y));
 			break;
 		}
@@ -277,10 +282,10 @@ bool TpicSpecialHandler::process (const char *prefix, istream &is, SpecialAction
 			drawLines(false, true, 0, actions);
 			break;
 		case cmd_id('d','a'): // as fp but draw dashed lines
-			drawLines(true, _fill >= 0, in.getDouble()*72.27, actions);
+			drawLines(true, _fill >= 0, in.getDouble()*72, actions);
 			break;
 		case cmd_id('d','t'): // as fp but draw dotted lines
-			drawLines(true, _fill >= 0, -in.getDouble()*72.27, actions);
+			drawLines(true, _fill >= 0, -in.getDouble()*72, actions);
 			break;
 		case cmd_id('s','p'): { // draw quadratic splines through recorded points
 			double ddist = in.getDouble();
@@ -288,20 +293,20 @@ bool TpicSpecialHandler::process (const char *prefix, istream &is, SpecialAction
 			break;
 		}
 		case cmd_id('a','r'): { // draw elliptical arc
-			double cx = in.getDouble()*PT;
-			double cy = in.getDouble()*PT;
-			double rx = in.getDouble()*PT;
-			double ry = in.getDouble()*PT;
+			double cx = in.getDouble()*mi2bp;
+			double cy = in.getDouble()*mi2bp;
+			double rx = in.getDouble()*mi2bp;
+			double ry = in.getDouble()*mi2bp;
 			double a1 = in.getDouble();
 			double a2 = in.getDouble();
 			drawArc(cx, cy, rx, ry, a1, a2, actions);
 			break;
 		}
 		case cmd_id('i','a'): { // fill elliptical arc
-			double cx = in.getDouble()*PT;
-			double cy = in.getDouble()*PT;
-			double rx = in.getDouble()*PT;
-			double ry = in.getDouble()*PT;
+			double cx = in.getDouble()*mi2bp;
+			double cy = in.getDouble()*mi2bp;
+			double rx = in.getDouble()*mi2bp;
+			double ry = in.getDouble()*mi2bp;
 			double a1 = in.getDouble();
 			double a2 = in.getDouble();
 			if (_fill < 0)
