@@ -2,7 +2,7 @@
 ** dvisvgm.cpp                                                          **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -19,9 +19,8 @@
 *************************************************************************/
 
 #include <config.h>
-#include <cmath>
+#include <clipper.hpp>
 #include <fstream>
-#include <iomanip>
 #include <iostream>
 #include <sstream>
 #include <string>
@@ -43,9 +42,10 @@
 #include "Message.h"
 #include "PageSize.h"
 #include "PSInterpreter.h"
+#include "PsSpecialHandler.h"
 #include "SignalHandler.h"
 #include "SpecialManager.h"
-#include "SVGOutputBase.h"
+#include "SVGOutput.h"
 #include "System.h"
 
 #ifdef __MSVC__
@@ -58,100 +58,13 @@ extern "C" {
 
 using namespace std;
 
-class SVGOutput : public SVGOutputBase
-{
-	public:
-		SVGOutput (const char *base=0, string pattern="", int zip_level=0)
-			: _path(base ? base : ""),
-			_pattern(pattern),
-			_stdout(base == 0),
-			_zipLevel(zip_level),
-			_page(-1),
-			_os(0) {}
-
-
-		~SVGOutput () {
-			delete _os;
-		}
-
-
-		/** Returns an output stream for the given page.
-		 *  @param[in] page number of current page
-		 *  @param[in] numPages total number of pages in the DVI file
-		 *  @return output stream for the given page */
-		ostream& getPageStream (int page, int numPages) const {
-			string fname = filename(page, numPages);
-			if (fname.empty()) {
-				delete _os;
-				_os = 0;
-				return cout;
-			}
-			if (page == _page)
-				return *_os;
-
-			_page = page;
-			delete _os;
-
-			if (_zipLevel > 0)
-				_os = new ogzstream(fname.c_str(), _zipLevel);
-			else
-				_os = new ofstream(fname.c_str());
-			if (!_os || !*_os) {
-				delete _os;
-				_os = 0;
-				throw MessageException("can't open file "+fname+" for writing");
-			}
-			return *_os;
-		}
-
-
-		/** Returns the name of the SVG file containing the given page.
-		 *  @param[in] page number of page */
-		string filename (int page, int numPages) const {
-			if (_stdout)
-				return "";
-			string fname = _pattern;
-			if (fname.empty())
-				fname = numPages > 1 ? "%f-%p" : "%f";
-			else if (numPages > 1 && fname.find("%p") == string::npos)
-				fname += FileSystem::isDirectory(fname.c_str()) ? "/%f-%p" : "-%p";
-
-			// replace pattern variables by their actual values
-			// %f: basename of the DVI file
-			// %p: current page number
-			ostringstream oss;
-			oss << setfill('0') << setw(max(2, int(1+log10((double)numPages)))) << page;
-			size_t pos=0;
-			while ((pos = fname.find('%', pos)) != string::npos && pos < fname.length()-1) {
-				switch (fname[pos+1]) {
-					case 'f': fname.replace(pos, 2, _path.basename());  pos += _path.basename().length(); break;
-					case 'p': fname.replace(pos, 2, oss.str()); pos += oss.str().length(); break;
-					default : ++pos;
-				}
-			}
-			FilePath outpath(fname, true);
-			if (outpath.suffix().empty())
-				outpath.suffix(_zipLevel > 0 ? "svgz" : "svg");
-			string apath = outpath.absolute();
-			string rpath = outpath.relative();
-			return apath.length() < rpath.length() ? apath : rpath;
-		}
-
-	private:
-		FilePath _path;
-		string _pattern;
-		bool _stdout;
-		int _zipLevel;
-		mutable int _page; // number of current page being written
-		mutable ostream *_os;
-};
 
 ////////////////////////////////////////////////////////////////////////////////
 
 static void show_help (const CommandLine &cmd) {
 	cout << PACKAGE_STRING "\n\n";
 	cmd.help(cmd.help_arg());
-   cout << "\nCopyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de> \n\n";
+	cout << "\nCopyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de> \n\n";
 }
 
 
@@ -192,18 +105,6 @@ static void set_libgs (CommandLine &args) {
 		Ghostscript::LIBGS_NAME = args.libgs_arg();
 	else if (getenv("LIBGS"))
 		Ghostscript::LIBGS_NAME = getenv("LIBGS");
-#ifdef MIKTEX
-	else {
-#ifdef __WIN64__
-		const char *gsdll = "mgsdll64.dll";
-#else
-		const char *gsdll = "mgsdll32.dll";
-#endif
-		// try to lookup the Ghostscript DLL coming with MiKTeX
-		if (const char *gsdll_path = FileFinder::lookup(gsdll))
-			Ghostscript::LIBGS_NAME = gsdll_path;
-	}
-#endif
 #endif
 }
 
@@ -217,36 +118,16 @@ static bool set_cache_dir (const CommandLine &args) {
 		else
 			Message::wstream(true) << "cache directory '" << args.cache_arg() << "' does not exist (caching disabled)\n";
 	}
-	else {
-		if (const char *userdir = FileSystem::userdir()) {
-			static string path = userdir;
-			path += "/.dvisvgm";
-			path = FileSystem::adaptPathSeperators(path);
-			const string cachepath = path+"/cache";
-			if (!FileSystem::exists(cachepath.c_str())) {
-				if (!FileSystem::exists(path.c_str()))
-					FileSystem::mkdir(cachepath.c_str());
-				else {
-					FileSystem::mkdir(cachepath.c_str());
-					// move existing cache files from former location to new one
-					vector<string> files;
-					FileSystem::collect(path.c_str(), files);
-					FORALL(files, vector<string>::iterator, it) {
-						if (it->at(0) == 'f' && it->length() > 4 && it->substr(it->length()-4, 4) == ".fgd") {
-							const char *fname = it->c_str()+1;
-							FileSystem::copy(path+"/"+fname, cachepath+"/"+fname, true);
-						}
-					}
-				}
-			}
-			path = cachepath;
-			PhysicalFont::CACHE_PATH = path.c_str();
-		}
-		if (args.cache_given() && args.cache_arg().empty()) {
-			cout << "cache directory: " << (PhysicalFont::CACHE_PATH ? PhysicalFont::CACHE_PATH : "(none)") << '\n';
-			FontCache::fontinfo(PhysicalFont::CACHE_PATH, cout, true);
-			return false;
-		}
+	else if (const char *userdir = FileSystem::userdir()) {
+		static string cachepath = userdir + string("/.dvisvgm/cache");
+		if (!FileSystem::exists(cachepath.c_str()))
+			FileSystem::mkdir(cachepath.c_str());
+		PhysicalFont::CACHE_PATH = cachepath.c_str();
+	}
+	if (args.cache_given() && args.cache_arg().empty()) {
+		cout << "cache directory: " << (PhysicalFont::CACHE_PATH ? PhysicalFont::CACHE_PATH : "(none)") << '\n';
+		FontCache::fontinfo(PhysicalFont::CACHE_PATH, cout, true);
+		return false;
 	}
 	return true;
 }
@@ -287,6 +168,7 @@ static void print_version (bool extended) {
 			oss << " (" TARGET_SYSTEM ")";
 		int len = oss.str().length();
 		oss << "\n" << string(len, '-') << "\n"
+			"clipper:     " << CLIPPER_VERSION "\n"
 			"freetype:    " << FontEngine::version() << "\n";
 
 		Ghostscript gs;
@@ -328,6 +210,11 @@ int main (int argc, char *argv[]) {
 	if (args.error())
 		return 1;
 
+	if (argc == 1 || args.help_given()) {
+		show_help(args);
+		return 0;
+	}
+
 	Message::COLORIZE = args.color_given();
 
 	try {
@@ -344,21 +231,14 @@ int main (int argc, char *argv[]) {
 		return 0;
 	}
 	if (args.list_specials_given()) {
-		SVGOutput out;
-		DVIToSVG dvisvg(cin, out);
-		if (const SpecialManager *sm = dvisvg.setProcessSpecials())
-			sm->writeHandlerInfo(cout);
+		DVIToSVG::setProcessSpecials();
+		SpecialManager::instance().writeHandlerInfo(cout);
 		return 0;
 	}
 
 	if (!set_cache_dir(args))
 		return 0;
 
-	if (argc == 1 || args.help_given()) {
-		show_help(args);
-		return 0;
-	}
-
 	if (argc > 1 && args.numFiles() < 1) {
 		Message::estream(true) << "no input file given\n";
 		return 1;
@@ -373,27 +253,35 @@ int main (int argc, char *argv[]) {
 		return 1;
 
 	if (args.progress_given()) {
-		DVIReader::COMPUTE_PAGE_LENGTH = args.progress_given();
-		DVIToSVGActions::PROGRESSBAR_DELAY = args.progress_arg();
+		DVIReader::COMPUTE_PROGRESS = args.progress_given();
+		SpecialActions::PROGRESSBAR_DELAY = args.progress_arg();
 	}
+	Color::SUPPRESS_COLOR_NAMES = !args.colornames_given();
 	SVGTree::CREATE_STYLE = !args.no_styles_given();
 	SVGTree::USE_FONTS = !args.no_fonts_given();
 	SVGTree::CREATE_USE_ELEMENTS = args.no_fonts_arg() < 1;
+	SVGTree::ZOOM_FACTOR = args.zoom_arg();
+	SVGTree::RELATIVE_PATH_CMDS = args.relative_given();
+	SVGTree::MERGE_CHARS = !args.no_merge_given();
 	DVIToSVG::TRACE_MODE = args.trace_all_given() ? (args.trace_all_arg() ? 'a' : 'm') : 0;
 	Message::LEVEL = args.verbosity_arg();
 	PhysicalFont::EXACT_BBOX = args.exact_given();
 	PhysicalFont::KEEP_TEMP_FILES = args.keep_given();
-	PhysicalFont::METAFONT_MAG = args.mag_arg();
+	PhysicalFont::METAFONT_MAG = max(1.0, args.mag_arg());
 	XMLString::DECIMAL_PLACES = max(0, min(6, args.precision_arg()));
 	if (!HtmlSpecialHandler::setLinkMarker(args.linkmark_arg()))
 		Message::wstream(true) << "invalid argument '"+args.linkmark_arg()+"' supplied for option --linkmark\n";
 	double start_time = System::time();
 	bool eps_given=false;
-#if defined(HAVE_LIBGS) || !defined(DISABLE_GS)
+#ifndef DISABLE_GS
 	eps_given = args.eps_given();
+	PsSpecialHandler::COMPUTE_CLIPPATHS_INTERSECTIONS = args.clipjoin_given();
+	PsSpecialHandler::SHADING_SEGMENT_OVERLAP = args.grad_overlap_given();
+	PsSpecialHandler::SHADING_SEGMENT_SIZE = max(1, args.grad_segments_arg());
+	PsSpecialHandler::SHADING_SIMPLIFY_DELTA = args.grad_simplify_arg();
 #endif
 	string inputfile = ensure_suffix(args.file(0), eps_given);
-	ifstream ifs(inputfile.c_str(), ios_base::binary|ios_base::in);
+	ifstream ifs(inputfile.c_str(), ios::binary|ios::in);
 	if (!ifs) {
 		Message::estream(true) << "can't open file '" << inputfile << "' for reading\n";
 		return 0;
@@ -401,7 +289,7 @@ int main (int argc, char *argv[]) {
 	try {
 		SVGOutput out(args.stdout_given() ? 0 : inputfile.c_str(), args.output_arg(), args.zip_given() ? args.zip_arg() : 0);
 		SignalHandler::instance().start();
-#if defined(HAVE_LIBGS) || !defined(DISABLE_GS)
+#ifndef DISABLE_GS
 		if (args.eps_given()) {
 			EPSToSVG eps2svg(inputfile, out);
 			eps2svg.convert();
@@ -412,13 +300,13 @@ int main (int argc, char *argv[]) {
 		else
 #endif
 		{
+			init_fontmap(args);
 			DVIToSVG dvi2svg(ifs, out);
 			const char *ignore_specials = args.no_specials_given() ? (args.no_specials_arg().empty() ? "*" : args.no_specials_arg().c_str()) : 0;
 			dvi2svg.setProcessSpecials(ignore_specials, true);
 			dvi2svg.setPageTransformation(get_transformation_string(args));
 			dvi2svg.setPageSize(args.bbox_arg());
 
-			init_fontmap(args);
 			pair<int,int> pageinfo;
 			dvi2svg.convert(args.page_arg(), &pageinfo);
 			Message::mstream().indent(0);
@@ -435,6 +323,7 @@ int main (int argc, char *argv[]) {
 		Message::estream() << "\nPostScript error: " << e.what() << '\n';
 	}
 	catch (SignalException &e) {
+		Message::wstream().clearline();
 		Message::wstream(true) << "execution interrupted by user\n";
 	}
 	catch (MessageException &e) {
