@@ -2,7 +2,7 @@
 ** ColorSpecialTest.cpp                                                 **
 **                                                                      **
 ** This file is part of dvisvgm -- the DVI to SVG converter             **
-** Copyright (C) 2005-2014 Martin Gieseking <martin.gieseking@uos.de>   **
+** Copyright (C) 2005-2015 Martin Gieseking <martin.gieseking@uos.de>   **
 **                                                                      **
 ** This program is free software; you can redistribute it and/or        **
 ** modify it under the terms of the GNU General Public License as       **
@@ -26,7 +26,7 @@
 class ColorSpecialTest : public ::testing::Test
 {
 	protected:
-		struct SetColor : SpecialEmptyActions
+		struct SetColor : EmptySpecialActions
 		{
 			SetColor () : color(0) {}
 			void setColor (const Color &c) {color = c;}
@@ -55,7 +55,7 @@ TEST_F(ColorSpecialTest, hsb) {
 TEST_F(ColorSpecialTest, cmyk) {
 	std::istringstream iss("cmyk 0.1 0.2 0.4 0.6");
 	handler.process(0, iss, &actions);
-	EXPECT_TRUE(actions.equals(0x4c3300));
+	EXPECT_TRUE(actions.equals(0x5c523d));
 }
 
 
